@@ -1,56 +1,7 @@
 """
 Creates core data types and functions which support all of brainbox.
 """
-from pathlib import Path
 import numpy as np
-import pandas as pd
-
-
-class Bunch(dict):
-    """A subclass of dictionary with an additional dot syntax."""
-
-    def __init__(self, *args, **kwargs):
-        import logging
-        logging.getLogger('ibllib').warning('Bunch has moved to ibl_util.util')  # iblrplate
-        super(Bunch, self).__init__(*args, **kwargs)
-        self.__dict__ = self
-
-    def copy(self):
-        """Return a new Bunch instance which is a copy of the current Bunch instance."""
-        return Bunch(super(Bunch, self).copy())
-
-    def to_df(self):
-        """Attempts to returns a pandas.DataFrame if all elements are arrays of the same length
-        Returns the original bunch if it can't"""
-        try:
-            return pd.DataFrame.from_dict(self)
-        except ValueError:
-            return self
-
-    def save(self, npz_file, compress=False):
-        """
-        Saves a npz file containing the arrays of the bunch.
-
-        :param npz_file: output file
-        :param compress: bool (False) use compression
-        :return: None
-        """
-        if compress:
-            np.savez_compressed(npz_file, **self)
-        else:
-            np.savez(npz_file, **self)
-
-    @staticmethod
-    def load(npz_file):
-        """
-        Loads a npz file containing the arrays of the bunch.
-
-        :param npz_file: output file
-        :return: Bunch
-        """
-        if not Path(npz_file).exists():
-            raise FileNotFoundError(f"{npz_file}")
-        return Bunch(np.load(npz_file))
 
 
 class TimeSeries(dict):
