@@ -5,34 +5,21 @@ from pathlib import Path, PurePosixPath
 import numpy as np
 import nrrd
 
-import boto3
-from botocore.config import Config
-from botocore import UNSIGNED
-
 from one.webclient import http_download_file
 import one.params
+import one.remote.aws as aws
 
 from iblutil.numerical import ismember
-from ibllib.atlas.regions import BrainRegions
+from ibllib.atlas.regions import BrainRegions, FranklinPaxinosRegions
 
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 ALLEN_CCF_LANDMARKS_MLAPDV_UM = {'bregma': np.array([5739, 5400, 332])}
+PAXINOS_CCF_LANDMARKS_MLAPDV_UM = {'bregma': np.array([5700, 4300 + 160, 330])}
+
 S3_BUCKET_IBL = 'ibl-brain-wide-map-public'
 
 
-def s3_download_public(bucket, object, destination):
-    """
-    downloads file from public bucket
-    :param bucket:
-    :param object:
-    :param destination:
-    :return:
-    """
-    boto3.client('s3', config=Config(signature_version=UNSIGNED)).download_file(
-        S3_BUCKET_IBL, object, str(destination))
-
-
 def cart2sph(x, y, z):
     """
     Converts cartesian to spherical Coordinates
@@ -246,7 +233,7 @@ class BrainAtlas:
         # create the coordinate transform object that maps volume indices to real world coordinates
         nxyz = np.array(self.image.shape)[self.dims2xyz]
         bc = BrainCoordinates(nxyz=nxyz, xyz0=(0, 0, 0), dxyz=dxyz)
-        self.bc = BrainCoordinates(nxyz=nxyz, xyz0=- bc.i2xyz(iorigin), dxyz=dxyz)
+        self.bc = BrainCoordinates(nxyz=nxyz, xyz0=-bc.i2xyz(iorigin), dxyz=dxyz)
 
         self.surface = None
         self.boundary = None
@@ -281,26 +268,26 @@ class BrainAtlas:
             self.srf_xyz = self.bc.i2xyz(np.c_[idx_srf[self.xyz2dims[0]], idx_srf[self.xyz2dims[1]],
                                                idx_srf[self.xyz2dims[2]]].astype(float))
 
-    def _lookup_inds(self, ixyz):
+    def _lookup_inds(self, ixyz, mode='raise'):
         """
         Performs a 3D lookup from volume indices ixyz to the image volume
         :param ixyz: [n, 3] array of indices in the mlapdv order
         :return: n array of flat indices
         """
         idims = np.split(ixyz[..., self.xyz2dims], [1, 2], axis=-1)
-        inds = np.ravel_multi_index(idims, self.bc.nxyz[self.xyz2dims])
+        inds = np.ravel_multi_index(idims, self.bc.nxyz[self.xyz2dims], mode=mode)
         return inds.squeeze()
 
-    def _lookup(self, xyz):
+    def _lookup(self, xyz, mode='raise'):
         """
         Performs a 3D lookup from real world coordinates to the flat indices in the volume
         defined in the BrainCoordinates object
         :param xyz: [n, 3] array of coordinates
         :return: n array of flat indices
         """
-        return self._lookup_inds(self.bc.xyz2i(xyz))
+        return self._lookup_inds(self.bc.xyz2i(xyz, mode=mode), mode=mode)
 
-    def get_labels(self, xyz, mapping='Allen', radius_um=None):
+    def get_labels(self, xyz, mapping=None, radius_um=None, mode='raise'):
         """
         Performs a 3D lookup from real world coordinates to the volume labels
         and return the regions ids according to the mapping
@@ -310,6 +297,8 @@ class BrainAtlas:
          of regions in a sphere of size radius around the coordinates.
         :return: n array of region ids
         """
+        mapping = mapping or self.regions.default_mapping
+
         if radius_um:
             nrx = int(np.ceil(radius_um / abs(self.bc.dx) / 1e6))
             nry = int(np.ceil(radius_um / abs(self.bc.dy) / 1e6))
@@ -326,15 +315,16 @@ class BrainAtlas:
             ilabs, counts = np.unique(cube[rcube <= radius_um], return_counts=True)
             return self.regions.id[ilabs], counts / np.sum(counts)
         else:
-            regions_indices = self._get_mapping(mapping=mapping)[self.label.flat[self._lookup(xyz)]]
+            regions_indices = self._get_mapping(mapping=mapping)[self.label.flat[self._lookup(xyz, mode=mode)]]
             return self.regions.id[regions_indices]
 
-    def _get_mapping(self, mapping='Allen'):
+    def _get_mapping(self, mapping=None):
         """
         Safe way to get mappings if nothing defined in regions.
         A mapping transforms from the full allen brain Atlas ids to the remapped ids
         new_ids = ids[mapping]
         """
+        mapping = mapping or self.regions.default_mapping
         if hasattr(self.regions, 'mappings'):
             return self.regions.mappings[mapping]
         else:
@@ -399,7 +389,7 @@ class BrainAtlas:
         depth = np.flipud(np.sort(sub_volume[:, ddim])[np.argsort(self.bc.lim(axis=ddim))])
         return tslice, width, height, depth
 
-    def plot_tilted_slice(self, xyz, axis, volume='image', cmap=None, ax=None, **kwargs):
+    def plot_tilted_slice(self, xyz, axis, volume='image', cmap=None, ax=None, sec_ax=False, **kwargs):
         """
         From line coordinates, extracts the tilted plane containing the line from the 3D volume
         :param xyz: np.array: points defining a probe trajectory in 3D space (xyz triplets)
@@ -438,15 +428,24 @@ class BrainAtlas:
         ax.set_xlabel(axis_labels[0])
         ax.set_ylabel(axis_labels[1])
         sec_ax.set_ylabel(axis_labels[2])
-        return ax
+        if sec_ax:
+            return ax, sec_ax
+        else:
+            return ax
 
     @staticmethod
-    def _plot_slice(im, extent, ax=None, cmap=None, **kwargs):
+    def _plot_slice(im, extent, ax=None, cmap=None, volume=None, **kwargs):
         if not ax:
             ax = plt.gca()
             ax.axis('equal')
         if not cmap:
             cmap = plt.get_cmap('bone')
+
+        if volume == 'boundary':
+            imb = np.zeros((*im.shape[:2], 4), dtype=np.uint8)
+            imb[im == 1] = np.array([0, 0, 0, 255])
+            im = imb
+
         ax.imshow(im, extent=extent, cmap=cmap, **kwargs)
         return ax
 
@@ -468,7 +467,7 @@ class BrainAtlas:
         return extent
 
     def slice(self, coordinate, axis, volume='image', mode='raise', region_values=None,
-              mapping="Allen", bc=None):
+              mapping=None, bc=None):
         """
         Get slice through atlas
 
@@ -544,12 +543,35 @@ class BrainAtlas:
         :param values:
         :return:
         """
-        boundary = np.diff(values, axis=0, append=0)
-        boundary = boundary + np.diff(values, axis=1, append=0)
+        boundary = np.abs(np.diff(values, axis=0, prepend=0))
+        boundary = boundary + np.abs(np.diff(values, axis=1, prepend=0))
+        boundary = boundary + np.abs(np.diff(values, axis=1, append=0))
+        boundary = boundary + np.abs(np.diff(values, axis=0, append=0))
+
         boundary[boundary != 0] = 1
+
         return boundary
 
-    def plot_cslice(self, ap_coordinate, volume='image', mapping='Allen', region_values=None, **kwargs):
+    def plot_slices(self, xyz, *args, **kwargs):
+        """
+        From a single coordinate, plots the 3 slices that intersect at this point in a single
+        matplotlib figure
+        :param xyz: mlapdv coordinate in m
+        :param args: arguments to be forwarded to plot slices
+        :param kwargs: keyword arguments to be forwarded to plot slices
+        :return: 2 by 2 array of axes
+        """
+        fig, axs = plt.subplots(2, 2)
+        self.plot_cslice(xyz[1], *args, ax=axs[0, 0], **kwargs)
+        self.plot_sslice(xyz[0], *args, ax=axs[0, 1], **kwargs)
+        self.plot_hslice(xyz[2], *args, ax=axs[1, 0], **kwargs)
+        xyz_um = xyz * 1e6
+        axs[0, 0].plot(xyz_um[0], xyz_um[2], 'g*')
+        axs[0, 1].plot(xyz_um[1], xyz_um[2], 'g*')
+        axs[1, 0].plot(xyz_um[0], xyz_um[1], 'g*')
+        return axs
+
+    def plot_cslice(self, ap_coordinate, volume='image', mapping=None, region_values=None, **kwargs):
         """
         Plot coronal slice through atlas at given ap_coordinate
 
@@ -571,9 +593,9 @@ class BrainAtlas:
         """
 
         cslice = self.slice(ap_coordinate, axis=1, volume=volume, mapping=mapping, region_values=region_values)
-        return self._plot_slice(np.moveaxis(cslice, 0, 1), extent=self.extent(axis=1), **kwargs)
+        return self._plot_slice(np.moveaxis(cslice, 0, 1), extent=self.extent(axis=1), volume=volume, **kwargs)
 
-    def plot_hslice(self, dv_coordinate, volume='image', mapping='Allen', region_values=None, **kwargs):
+    def plot_hslice(self, dv_coordinate, volume='image', mapping=None, region_values=None, **kwargs):
         """
         Plot horizontal slice through atlas at given dv_coordinate
 
@@ -595,9 +617,9 @@ class BrainAtlas:
         """
 
         hslice = self.slice(dv_coordinate, axis=2, volume=volume, mapping=mapping, region_values=region_values)
-        return self._plot_slice(hslice, extent=self.extent(axis=2), **kwargs)
+        return self._plot_slice(hslice, extent=self.extent(axis=2), volume=volume, **kwargs)
 
-    def plot_sslice(self, ml_coordinate, volume='image', mapping='Allen', region_values=None, **kwargs):
+    def plot_sslice(self, ml_coordinate, volume='image', mapping=None, region_values=None, **kwargs):
         """
         Plot sagittal slice through atlas at given ml_coordinate
 
@@ -619,9 +641,9 @@ class BrainAtlas:
         """
 
         sslice = self.slice(ml_coordinate, axis=0, volume=volume, mapping=mapping, region_values=region_values)
-        return self._plot_slice(np.swapaxes(sslice, 0, 1), extent=self.extent(axis=0), **kwargs)
+        return self._plot_slice(np.swapaxes(sslice, 0, 1), extent=self.extent(axis=0), volume=volume, **kwargs)
 
-    def plot_top(self, volume='annotation', mapping='Allen', region_values=None, ax=None, **kwargs):
+    def plot_top(self, volume='annotation', mapping=None, region_values=None, ax=None, **kwargs):
         """
         Plot top view of atlas
         :param volume:
@@ -659,7 +681,7 @@ class BrainAtlas:
         elif volume == 'boundary':
             im = self.compute_boundaries(regions)
 
-        return self._plot_slice(im, self.extent(axis=2), ax=ax, **kwargs)
+        return self._plot_slice(im, self.extent(axis=2), ax=ax, volume=volume, **kwargs)
 
 
 @dataclass
@@ -862,7 +884,7 @@ class Insertion:
         # In some cases the nearest two intersection points are not the top and bottom of brain
         # So we find all intersection points that fall within one voxel and take the one with
         # highest dV to be entry and lowest dV to be exit
-        idx_lim = np.sum(distance[dist_sort] * 1e6 < brain_atlas.res_um)
+        idx_lim = np.sum(distance[dist_sort] * 1e6 < np.max(brain_atlas.res_um))
         dist_lim = dist_sort[0:idx_lim]
         z_val = brain_atlas.srf_xyz[dist_lim, 2]
         if surface == 'top':
@@ -985,18 +1007,22 @@ class AllenAtlas(BrainAtlas):
             volume = np.load(file_volume)['arr_0']
         return volume
 
-    def xyz2ccf(self, xyz, ccf_order='mlapdv'):
+    def xyz2ccf(self, xyz, ccf_order='mlapdv', mode='raise'):
         """
         Converts coordinates to the CCF coordinates, which is assumed to be the cube indices
         times the spacing.
         :param xyz: mlapdv coordinates in meters, origin Bregma
         :param ccf_order: order that you want values returned 'mlapdv' (ibl) or 'apdvml'
         (Allen mcc vertices)
+        :param mode: {‘raise’, 'clip', 'wrap'} determines what to do when determined index lies outside the atlas volume
+                     'raise' will raise a ValueError
+                     'clip' will replace the index with the closest index inside the volume
+                     'wrap' will wrap around to the other side of the volume. This is only here for legacy reasons
         :return: coordinates in CCF space um, origin is the front left top corner of the data
         volume, order determined by ccf_order
         """
         ordre = self._ccf_order(ccf_order)
-        ccf = self.bc.xyz2i(xyz, round=False) * float(self.res_um)
+        ccf = self.bc.xyz2i(xyz, round=False, mode=mode) * float(self.res_um)
         return ccf[..., ordre]
 
     def ccf2xyz(self, ccf, ccf_order='mlapdv'):
@@ -1032,6 +1058,22 @@ class AllenAtlas(BrainAtlas):
         else:
             ValueError("ccf_order needs to be either 'mlapdv' or 'apdvml'")
 
+    def compute_regions_volume(self):
+        """
+        Sums the number of voxels in the labels volume for each region.
+        Then compute volumes for all of the levels of hierarchy in cubic mm.
+        :return:
+        """
+        nr = self.regions.id.shape[0]
+        count = np.bincount(self.label.flatten(), minlength=nr)
+        self.regions.compute_hierarchy()
+        self.regions.volume = np.zeros_like(count)
+        for i in np.arange(nr):
+            if count[i] == 0:
+                continue
+            self.regions.volume[np.unique(self.regions.hierarchy[:, i])] += count[i]
+        self.regions.volume = self.regions.volume * (self.res_um / 1e3) ** 3
+
 
 def NeedlesAtlas(*args, **kwargs):
     """
@@ -1047,6 +1089,22 @@ def NeedlesAtlas(*args, **kwargs):
     return AllenAtlas(*args, **kwargs)
 
 
+def MRITorontoAtlas(*args, **kwargs):
+    """
+    Instantiates an atlas.BrainAtlas corresponding to the Allen CCF at the given resolution
+    using the IBL Bregma and coordinate system. The MRI Toronto atlas defines a stretch along AP
+    a squeeze along DV *and* a squeeze along ML. These are based on 12 p65 mice MRIs averaged.
+    See: https://www.nature.com/articles/s41467-018-04921-2 DB has access to the dataset.
+    :param res_um: 10, 25 or 50 um
+    :return: atlas.BrainAtlas
+    """
+    ML_SCALE = 0.952
+    DV_SCALE = 0.885  # multiplicative factor on DV dimension, determined from MRI->CCF transform
+    AP_SCALE = 1.031  # multiplicative factor on AP dimension
+    kwargs['scaling'] = np.array([ML_SCALE, AP_SCALE, DV_SCALE])
+    return AllenAtlas(*args, **kwargs)
+
+
 def _download_atlas_allen(file_image, FLAT_IRON_ATLAS_REL_PATH, par):
     """
     © 2015 Allen Institute for Brain Science. Allen Mouse Brain Atlas (2015)
@@ -1104,7 +1162,7 @@ class FlatMap(AllenAtlas):
         file_flatmap = self._get_cache_dir().joinpath(f'{self.name}_{self.res_um}.nrrd')
         if not file_flatmap.exists():
             file_flatmap.parent.mkdir(exist_ok=True, parents=True)
-            s3_download_public(S3_BUCKET_IBL, f'atlas/{file_flatmap.name}', str(file_flatmap))
+            aws.s3_download_file(f'atlas/{file_flatmap.name}', file_flatmap)
         self.flatmap, _ = nrrd.read(file_flatmap)
 
     def plot_flatmap(self, depth=0, volume='annotation', mapping='Allen', region_values=None, ax=None, **kwargs):
@@ -1135,8 +1193,82 @@ class FlatMap(AllenAtlas):
         if not ax:
             ax = plt.gca()
 
-        return self._plot_slice(im, self.extent_flmap(), ax=ax, **kwargs)
+        return self._plot_slice(im, self.extent_flmap(), ax=ax, volume=volume, **kwargs)
 
     def extent_flmap(self):
         extent = np.r_[0, self.flatmap.shape[1], 0, self.flatmap.shape[0]]
         return extent
+
+
+class FranklinPaxinosAtlas(BrainAtlas):
+    """
+    Instantiates an atlas.BrainAtlas corresponding to the Allen CCF at the given resolution
+    using the IBL Bregma and coordinate system
+    """
+
+    def __init__(self, res_um=np.array([10, 100, 10]), scaling=np.array([1, 1, 1]), mock=False, hist_path=None):
+        """
+        :param res_um: 10, 25 or 50 um
+        :param scaling: scale factor along ml, ap, dv for squeeze and stretch ([1, 1, 1])
+        :param mock: for testing purpose
+        :param hist_path
+        :return: atlas.BrainAtlas
+        """
+        # TODO interpolate?
+        par = one.params.get(silent=True)
+        FLAT_IRON_ATLAS_REL_PATH = PurePosixPath('histology', 'ATLAS', 'Needles', 'FranklinPaxinos')
+        LUT_VERSION = "v01"  # version 01 is the lateralized version
+        regions = FranklinPaxinosRegions()
+        xyz2dims = np.array([1, 0, 2])  # this is the c-contiguous ordering
+        dims2xyz = np.array([1, 0, 2])
+        # we use Bregma as the origin
+        self.res_um = res_um
+        ibregma = (PAXINOS_CCF_LANDMARKS_MLAPDV_UM['bregma'] / self.res_um)
+        dxyz = self.res_um * 1e-6 * np.array([1, -1, -1]) * scaling
+        if mock:
+            image, label = [np.zeros((528, 456, 320), dtype=np.int16) for _ in range(2)]
+            label[:, :, 100:105] = 1327  # lookup index for retina, id 304325711 (no id 1327)
+        else:
+            path_atlas = Path(par.CACHE_DIR).joinpath(FLAT_IRON_ATLAS_REL_PATH)
+            file_image = hist_path or path_atlas.joinpath(f'average_template_{res_um[0]}_{res_um[1]}_{res_um[2]}.npz')
+            # # get the image volume
+            if not file_image.exists():
+                path_atlas.mkdir(exist_ok=True, parents=True)
+                aws.s3_download_file(f'atlas/FranklinPaxinos/{file_image.name}', str(file_image))
+            # # get the remapped label volume
+            file_label = path_atlas.joinpath(f'annotation_{res_um[0]}_{res_um[1]}_{res_um[2]}.npz')
+            if not file_label.exists():
+                path_atlas.mkdir(exist_ok=True, parents=True)
+                aws.s3_download_file(f'atlas/FranklinPaxinos/{file_label.name}', str(file_label))
+
+            file_label_remap = path_atlas.joinpath(f'annotation_{res_um[0]}_{res_um[1]}_{res_um[2]}_lut_{LUT_VERSION}.npz')
+
+            if not file_label_remap.exists():
+                label = self._read_volume(file_label).astype(dtype=np.int32)
+                _logger.info("computing brain atlas annotations lookup table")
+                # lateralize atlas: for this the regions of the left hemisphere have primary
+                # keys opposite to to the normal ones
+                lateral = np.zeros(label.shape[xyz2dims[0]])
+                lateral[int(np.floor(ibregma[0]))] = 1
+                lateral = np.sign(np.cumsum(lateral)[np.newaxis, :, np.newaxis] - 0.5)
+                label = label * lateral.astype(np.int32)
+                _, im = ismember(label, regions.id)
+                label = np.reshape(im.astype(np.uint16), label.shape)
+                np.savez_compressed(file_label_remap, label)
+                _logger.info(f"Cached remapping file {file_label_remap} ...")
+            # loads the files
+            label = self._read_volume(file_label_remap)
+            image = self._read_volume(file_image)
+
+        super().__init__(image, label, dxyz, regions, ibregma,
+                         dims2xyz=dims2xyz, xyz2dims=xyz2dims)
+
+    @staticmethod
+    def _read_volume(file_volume):
+        if file_volume.suffix == '.nrrd':
+            volume, _ = nrrd.read(file_volume, index_order='C')  # ml, dv, ap
+            # we want the coronal slice to be the most contiguous
+            volume = np.transpose(volume, (2, 0, 1))  # image[iap, iml, idv]
+        elif file_volume.suffix == '.npz':
+            volume = np.load(file_volume)['arr_0']
+        return volume
