@@ -3,18 +3,24 @@ Module that hold techniques to project the brain volume onto 2D images for visua
 """
 from functools import lru_cache
 import logging
+import json
 
-import pandas as pd
 import numpy as np
 from scipy.interpolate import interp1d
 import matplotlib.pyplot as plt
+import matplotlib.colors
+from matplotlib import cm
 
 from iblutil.numerical import ismember
 from iblutil.util import Bunch
 from iblutil.io.hashfile import md5
-from ibllib.atlas.atlas import AllenAtlas, S3_BUCKET_IBL, BrainRegions, s3_download_public
+import one.remote.aws as aws
 
-_logger = logging.getLogger(__file__)
+from ibllib.atlas.atlas import AllenAtlas, BrainRegions
+from ibllib.atlas.plots import plot_polygon, plot_polygon_with_hole, coords_for_poly_hole
+
+
+_logger = logging.getLogger(__name__)
 
 
 @lru_cache(maxsize=1, typed=False)
@@ -123,14 +129,123 @@ def swanson(filename="swanson2allen.npz"):
     npz_file = AllenAtlas._get_cache_dir().joinpath(filename)
     if not npz_file.exists() or md5(npz_file) in OLD_MD5:
         npz_file.parent.mkdir(exist_ok=True, parents=True)
-        _logger.info(f'downloading swanson image from {S3_BUCKET_IBL} s3 bucket...')
-        s3_download_public(S3_BUCKET_IBL, f'atlas/{npz_file.name}', str(npz_file))
+        _logger.info(f'downloading swanson image from {aws.S3_BUCKET_IBL} s3 bucket...')
+        aws.s3_download_file(f'atlas/{npz_file.name}', npz_file)
     s2a = np.load(npz_file)['swanson2allen']  # inds contains regions ids
     return s2a
 
 
+def swanson_json(filename="swansonpaths.json"):
+
+    OLD_MD5 = ['f848783954883c606ca390ceda9e37d2']
+
+    json_file = AllenAtlas._get_cache_dir().joinpath(filename)
+    if not json_file.exists() or md5(json_file) in OLD_MD5:
+        json_file.parent.mkdir(exist_ok=True, parents=True)
+        _logger.info(f'downloading swanson paths from {aws.S3_BUCKET_IBL} s3 bucket...')
+        aws.s3_download_file(f'atlas/{json_file.name}', json_file)
+
+    with open(json_file) as f:
+        sw_json = json.load(f)
+
+    return sw_json
+
+
+def plot_swanson_vector(acronyms=None, values=None, ax=None, hemisphere=None, br=None, orientation='landscape',
+                        empty_color='silver', vmin=None, vmax=None, cmap='viridis', **kwargs):
+
+    br = BrainRegions() if br is None else br
+    br.compute_hierarchy()
+
+    if ax is None:
+        fig, ax = plt.subplots()
+        ax.set_axis_off()
+
+    if acronyms is not None:
+        ibr, vals = br.propagate_down(acronyms, values)
+        colormap = cm.get_cmap(cmap)
+        vmin = vmin or np.nanmin(vals)
+        vmax = vmax or np.nanmax(vals)
+        norm = matplotlib.colors.Normalize(vmin=vmin, vmax=vmax)
+        rgba_color = colormap(norm(vals), bytes=True)
+
+    sw = swanson()
+    sw_json = swanson_json()
+
+    for i, reg in enumerate(sw_json):
+
+        if acronyms is None:
+            color = br.rgba[br.mappings['Swanson'][reg['thisID']]] / 255
+        else:
+            idx = np.where(ibr == reg['thisID'])[0]
+            color = rgba_color[idx[0]] / 255 if len(idx) > 0 else empty_color
+
+        coords = reg['coordsReg']
+
+        if reg['hole']:
+            vertices, codes = coords_for_poly_hole(coords)
+            if orientation == 'portrait':
+                vertices[:, [0, 1]] = vertices[:, [1, 0]]
+                plot_polygon_with_hole(ax, vertices, codes, color, **kwargs)
+                if hemisphere is not None:
+                    color_inv = color if hemisphere == 'mirror' else empty_color
+                    vertices_inv = np.copy(vertices)
+                    vertices_inv[:, 0] = -1 * vertices_inv[:, 0] + (sw.shape[0] * 2)
+                    plot_polygon_with_hole(ax, vertices_inv, codes, color_inv, **kwargs)
+            else:
+                plot_polygon_with_hole(ax, vertices, codes, color, **kwargs)
+                if hemisphere is not None:
+                    color_inv = color if hemisphere == 'mirror' else empty_color
+                    vertices_inv = np.copy(vertices)
+                    vertices_inv[:, 1] = -1 * vertices_inv[:, 1] + (sw.shape[0] * 2)
+                    plot_polygon_with_hole(ax, vertices_inv, codes, color_inv, **kwargs)
+        else:
+            coords = [coords] if type(coords) == dict else coords
+            for c in coords:
+
+                if orientation == 'portrait':
+                    xy = np.c_[c['y'], c['x']]
+                    plot_polygon(ax, xy, color, **kwargs)
+                    if hemisphere is not None:
+                        color_inv = color if hemisphere == 'mirror' else empty_color
+                        xy_inv = np.copy(xy)
+                        xy_inv[:, 0] = -1 * xy_inv[:, 0] + (sw.shape[0] * 2)
+                        plot_polygon(ax, xy_inv, color_inv, **kwargs)
+                else:
+                    xy = np.c_[c['x'], c['y']]
+                    plot_polygon(ax, xy, color, **kwargs)
+                    if hemisphere is not None:
+                        color_inv = color if hemisphere == 'mirror' else empty_color
+                        xy_inv = np.copy(xy)
+                        xy_inv[:, 1] = -1 * xy_inv[:, 1] + (sw.shape[0] * 2)
+                        plot_polygon(ax, xy_inv, color_inv, **kwargs)
+
+    if orientation == 'portrait':
+        ax.set_ylim(0, sw.shape[1])
+        if hemisphere is None:
+            ax.set_xlim(0, sw.shape[0])
+        else:
+            ax.set_xlim(0, 2 * sw.shape[0])
+    else:
+        ax.set_xlim(0, sw.shape[1])
+        if hemisphere is None:
+            ax.set_ylim(0, sw.shape[0])
+        else:
+            ax.set_ylim(0, 2 * sw.shape[0])
+
+    def format_coord(x, y):
+        ind = sw[int(y), int(x)]
+        ancestors = br.ancestors(br.id[ind])['acronym']
+        return f'sw-{ind}, x={x:1.4f}, y={y:1.4f}, aid={br.id[ind]}-{br.acronym[ind]} \n {ancestors}'
+
+    ax.format_coord = format_coord
+
+    ax.invert_yaxis()
+    ax.set_aspect('equal')
+
+
 def plot_swanson(acronyms=None, values=None, ax=None, hemisphere=None, br=None,
-                 orientation='landscape', annotate=False, **kwargs):
+                 orientation='landscape', annotate=False, empty_color='silver', **kwargs):
     """
     Displays the 2D image corresponding to the swanson flatmap.
     This case is different from the others in the sense that only a region maps to another regions, there
@@ -142,11 +257,14 @@ def plot_swanson(acronyms=None, values=None, ax=None, hemisphere=None, br=None,
     :param ax: matplotlib axis object to plot onto
     :param orientation: 'landscape' (default) or 'portrait'
     :param annotate: (False) if True, labels regions with acronyms
+    :param empty_color: (grey) matplotlib color code or rgb_a int8 tuple defining the filling
+     of brain regions not provided. Defaults to 'silver'
     :param kwargs: arguments for imshow
     :return:
     """
     mapping = 'Swanson'
     br = BrainRegions() if br is None else br
+    br.compute_hierarchy()
     s2a = swanson()
     # both hemishpere
     if hemisphere == 'both':
@@ -162,26 +280,9 @@ def plot_swanson(acronyms=None, values=None, ax=None, hemisphere=None, br=None,
     if acronyms is None:
         regions = br.mappings[mapping][s2a]
         im = br.rgba[regions]
+        iswan = None
     else:
-        user_aids = br.parse_acronyms_argument(acronyms)
-        # if the user provided inputs are higher level than swanson propagate down
-        swaids = br.id[np.unique(s2a)]
-        maids = np.setdiff1d(user_aids, swaids)  # those are the indices not in Swanson
-        for i, maid in enumerate(maids):
-            if maid <= 1:
-                continue
-            childs_in_sw = np.intersect1d(br.descendants(maid)['id'][1:], swaids)
-            if childs_in_sw.size > 0:
-                user_aids = np.r_[user_aids, childs_in_sw]
-                values = np.r_[values, values[i] + childs_in_sw * 0]
-        # the user may have input non-unique regions
-        df = pd.DataFrame(dict(aid=user_aids, value=values)).groupby('aid').mean()
-        aids, vals = (df.index.values, df['value'].values)
-        # apply mapping and perform another round of aggregation
-        _, _, ibr = np.intersect1d(aids, br.id, return_indices=True)
-        ibr = br.mappings['Swanson-lr'][ibr]
-        df = pd.DataFrame(dict(ibr=ibr, value=vals)).groupby('ibr').mean()
-        ibr, vals = (df.index.values, df['value'].values)
+        ibr, vals = br.propagate_down(acronyms, values)
         # we now have the mapped regions and aggregated values, map values onto swanson map
         iswan, iv = ismember(s2a, ibr)
         im = np.zeros_like(s2a, dtype=np.float32)
@@ -193,6 +294,9 @@ def plot_swanson(acronyms=None, values=None, ax=None, hemisphere=None, br=None,
     ax.imshow(im, **kwargs)
     # overlay the boundaries if value plot
     imb = np.zeros((*s2a.shape[:2], 4), dtype=np.uint8)
+    # fill in the empty regions with the blank regions colours if necessary
+    if iswan is not None:
+        imb[~iswan] = (np.array(matplotlib.colors.to_rgba(empty_color)) * 255).astype('uint8')
     imb[s2a == 0] = 255
     # imb[s2a == 1] = np.array([167, 169, 172, 255])
     imb[s2a == 1] = np.array([0, 0, 0, 255])
@@ -202,8 +306,9 @@ def plot_swanson(acronyms=None, values=None, ax=None, hemisphere=None, br=None,
 
     # provides the mean to see the region on axis
     def format_coord(x, y):
-        acronym = br.acronym[s2a[int(y), int(x)]]
-        return f'x={x:1.4f}, y={x:1.4f}, {acronym}'
+        ind = s2a[int(y), int(x)]
+        ancestors = br.ancestors(br.id[ind])['acronym']
+        return f'sw-{ind}, x={x:1.4f}, y={y:1.4f}, aid={br.id[ind]}-{br.acronym[ind]} \n {ancestors}'
 
     ax.format_coord = format_coord
     return ax
