@@ -7,10 +7,11 @@ import pandas as pd
 from iblutil.util import Bunch
 from iblutil.numerical import ismember
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 # 'Beryl' is the name given to an atlas containing a subset of the most relevant allen annotations
 FILE_MAPPINGS = str(Path(__file__).parent.joinpath('mappings.pqt'))
-FILE_REGIONS = str(Path(__file__).parent.joinpath('allen_structure_tree.csv'))
+ALLEN_FILE_REGIONS = str(Path(__file__).parent.joinpath('allen_structure_tree.csv'))
+FRANKLIN_FILE_REGIONS = str(Path(__file__).parent.joinpath('franklin_paxinos_structure_tree.csv'))
 
 
 @dataclass
@@ -23,40 +24,6 @@ class _BrainRegions:
     parent: np.ndarray
     order: np.uint16
 
-
-class BrainRegions(_BrainRegions):
-    """
-    ibllib.atlas.regions.BrainRegions(brainmap='Allen')
-    The Allen atlas ids are kept intact but lateralized as follows: labels are duplicated
-     and ids multiplied by -1, with the understanding that left hemisphere regions have negative
-     ids.
-    """
-    def __init__(self):
-        df_regions = pd.read_csv(FILE_REGIONS)
-        # lateralize
-        df_regions_left = df_regions.iloc[np.array(df_regions.id > 0), :].copy()
-        df_regions_left['id'] = - df_regions_left['id']
-        df_regions_left['parent_structure_id'] = - df_regions_left['parent_structure_id']
-        df_regions_left['name'] = df_regions_left['name'].apply(lambda x: x + ' (left)')
-        df_regions = pd.concat((df_regions, df_regions_left), axis=0)
-        # converts colors to RGB uint8 array
-        c = np.uint32(df_regions.color_hex_triplet.map(
-            lambda x: int(x, 16) if isinstance(x, str) else 256 ** 3 - 1))
-        c = np.flip(np.reshape(c.view(np.uint8), (df_regions.id.size, 4))[:, :3], 1)
-        c[0, :] = 0  # set the void region to black
-        # creates the BrainRegion instance
-        super().__init__(id=df_regions.id.to_numpy(),
-                         name=df_regions.name.to_numpy(),
-                         acronym=df_regions.acronym.to_numpy(),
-                         rgb=c,
-                         level=df_regions.depth.to_numpy().astype(np.uint16),
-                         parent=df_regions.parent_structure_id.to_numpy(),
-                         order=df_regions.graph_order.to_numpy().astype(np.uint16))
-        # mappings are indices not ids: they range from 0 to n regions -1
-        mappings = pd.read_parquet(FILE_MAPPINGS)
-        self.mappings = {k: mappings[k].to_numpy() for k in mappings}
-        self.n_lr = int((len(self.id) - 1) / 2)
-
     @property
     def rgba(self):
         rgba = np.c_[self.rgb, self.rgb[:, 0] * 0 + 255]
@@ -74,28 +41,6 @@ class BrainRegions(_BrainRegions):
         # Then right hemisphere
         orders[2::2] = np.arange(self.n_lr) + 1
 
-    def _compute_mappings(self):
-        """
-        Recomputes the mapping indices for all mappings
-        This is left mainly as a reference for adding future mappings as this take a few seconds
-        to execute. In production,we use the MAPPING_FILES pqt to avoid recompuing at each \
-        instantiation
-        """
-        beryl = np.load(Path(__file__).parent.joinpath('beryl.npy'))
-        cosmos = np.load(Path(__file__).parent.joinpath('cosmos.npy'))
-        swanson = np.load(Path(__file__).parent.joinpath('swanson_regions.npy'))
-        self.mappings = {
-            'Allen': self._mapping_from_regions_list(np.unique(np.abs(self.id)), lateralize=False),
-            'Allen-lr': np.arange(self.id.size),
-            'Beryl': self._mapping_from_regions_list(beryl, lateralize=False),
-            'Beryl-lr': self._mapping_from_regions_list(beryl, lateralize=True),
-            'Cosmos': self._mapping_from_regions_list(cosmos, lateralize=False),
-            'Cosmos-lr': self._mapping_from_regions_list(cosmos, lateralize=True),
-            'Swanson': self._mapping_from_regions_list(swanson, lateralize=False),
-            'Swanson-lr': self._mapping_from_regions_list(swanson, lateralize=True),
-        }
-        pd.DataFrame(self.mappings).to_parquet(FILE_MAPPINGS)
-
     def get(self, ids) -> Bunch:
         """
         Get a bunch of the name/id
@@ -178,6 +123,26 @@ class BrainRegions(_BrainRegions):
         leaves = np.setxor1d(self.id, self.parent)
         return self.get(np.int64(leaves[~np.isnan(leaves)]))
 
+    def propagate_down(self, acronyms, values):
+        """
+        This function remaps a set of user specified acronyms and values to the
+        swanson map, by filling down the child nodes when higher up values are
+        provided.
+        :param acronyms: list or array of allen ids or acronyms
+        :param values: list or array of associated values
+        :return:
+        """
+        user_aids = self.parse_acronyms_argument(acronyms)
+        _, user_indices = ismember(user_aids, self.id)
+        self.compute_hierarchy()
+        ia, ib = ismember(self.hierarchy, user_indices)
+        v = np.zeros_like(ia, dtype=np.float64) * np.NaN
+        v[ia] = values[ib]
+        all_values = np.nanmedian(v, axis=0)
+        indices = np.where(np.any(ia, axis=0))[0]
+        all_values = all_values[indices]
+        return indices, all_values
+
     def _mapping_from_regions_list(self, new_map, lateralize=False):
         """
         From a vector of regions id, creates a mapping such as
@@ -208,18 +173,7 @@ class BrainRegions(_BrainRegions):
             mapind = mapind[iregion]
         return mapind
 
-    def remap(self, region_ids, source_map='Allen', target_map='Beryl'):
-        """
-        Remap atlas regions ids from source map to target map
-        :param region_ids: atlas ids to map
-        :param source_map: map name which original region_ids are in
-        :param target_map: map name onto which to map
-        :return:
-        """
-        _, inds = ismember(region_ids, self.id[self.mappings[source_map]])
-        return self.id[self.mappings[target_map][inds]]
-
-    def acronym2acronym(self, acronym, mapping='Allen'):
+    def acronym2acronym(self, acronym, mapping=None):
         """
         Remap acronyms onto mapping
 
@@ -227,10 +181,11 @@ class BrainRegions(_BrainRegions):
         :param mapping: target map to remap acronyms
         :return: array of remapped acronyms
         """
+        mapping = mapping or self.default_mapping
         inds = self._find_inds(acronym, self.acronym)
         return self.acronym[self.mappings[mapping]][inds]
 
-    def acronym2id(self, acronym, mapping='Allen', hemisphere=None):
+    def acronym2id(self, acronym, mapping=None, hemisphere=None):
         """
         Convert acronyms to atlas ids and remap
 
@@ -239,10 +194,11 @@ class BrainRegions(_BrainRegions):
         :param hemisphere: which hemisphere to return atlas ids for, options left or right
         :return: array of remapped atlas ids
         """
+        mapping = mapping or self.default_mapping
         inds = self._find_inds(acronym, self.acronym)
         return self.id[self.mappings[mapping]][self._filter_lr(inds, mapping, hemisphere)]
 
-    def acronym2index(self, acronym, mapping='Allen', hemisphere=None):
+    def acronym2index(self, acronym, mapping=None, hemisphere=None):
         """
         Convert acronym to index and remap
         :param acronym:
@@ -250,6 +206,7 @@ class BrainRegions(_BrainRegions):
         :param hemisphere:
         :return: array of remapped acronyms and list of indexes for each acronnym
         """
+        mapping = mapping or self.default_mapping
         acronym = self.acronym2acronym(acronym, mapping=mapping)
         index = list()
         for id in acronym:
@@ -258,7 +215,7 @@ class BrainRegions(_BrainRegions):
 
         return acronym, index
 
-    def id2acronym(self, atlas_id, mapping='Allen'):
+    def id2acronym(self, atlas_id, mapping=None):
         """
         Convert atlas id to acronym and remap
 
@@ -266,6 +223,7 @@ class BrainRegions(_BrainRegions):
         :param mapping: target map to remap acronyms
         :return: array of remapped acronyms
         """
+        mapping = mapping or self.default_mapping
         inds = self._find_inds(atlas_id, self.id)
         return self.acronym[self.mappings[mapping]][inds]
 
@@ -298,7 +256,7 @@ class BrainRegions(_BrainRegions):
 
         return atlas_id, index
 
-    def index2acronym(self, index, mapping='Allen'):
+    def index2acronym(self, index, mapping=None):
         """
         Convert index to acronym and remap
 
@@ -306,10 +264,11 @@ class BrainRegions(_BrainRegions):
         :param mapping:
         :return:
         """
+        mapping = mapping or self.default_mapping
         inds = self.acronym[self.mappings[mapping]][index]
         return inds
 
-    def index2id(self, index, mapping='Allen'):
+    def index2id(self, index, mapping=None):
         """
         Convert index to atlas id and remap
 
@@ -317,6 +276,7 @@ class BrainRegions(_BrainRegions):
         :param mapping:
         :return:
         """
+        mapping = mapping or self.default_mapping
         inds = self.id[self.mappings[mapping]][index]
         return inds
 
@@ -381,6 +341,171 @@ class BrainRegions(_BrainRegions):
         return user_aids
 
 
+class FranklinPaxinosRegions(_BrainRegions):
+    def __init__(self):
+        df_regions = pd.read_csv(FRANKLIN_FILE_REGIONS)
+        # get rid of nan values, there are rows that are in Allen but are not in the Franklin Paxinos atlas
+        df_regions = df_regions[~df_regions['Structural ID'].isna()]
+        # add in root
+        root = [{'Structural ID': int(997), 'Franklin-Paxinos Full name': 'root', 'Franklin-Paxinos abbreviation': 'root',
+                 'structure Order': 50, 'red': 255, 'green': 255, 'blue': 255, 'Allen Full name': 'root',
+                 'Allen abbreviation': 'root'}]
+        df_regions = pd.concat([pd.DataFrame(root), df_regions], ignore_index=True)
+
+        allen_regions = pd.read_csv(ALLEN_FILE_REGIONS)
+
+        # Find the level of acronyms that are the same as Allen
+        a, b = ismember(df_regions['Allen abbreviation'].values, allen_regions['acronym'].values)
+        level = allen_regions['depth'].values[b]
+        df_regions['level'] = np.full(len(df_regions), np.nan)
+        df_regions['allen level'] = np.full(len(df_regions), np.nan)
+        df_regions.loc[a, 'level'] = level
+        df_regions.loc[a, 'allen level'] = level
+
+        nan_idx = np.where(df_regions['Allen abbreviation'].isna())[0]
+        df_regions.loc[nan_idx, 'Allen abbreviation'] = df_regions['Franklin-Paxinos abbreviation'].values[nan_idx]
+        df_regions.loc[nan_idx, 'Allen Full name'] = df_regions['Franklin-Paxinos Full name'].values[nan_idx]
+
+        # Now fill in the nan values with one level up from their parents we need to this multiple times
+        while np.sum(np.isnan(df_regions['level'].values)) > 0:
+            nan_loc = np.isnan(df_regions['level'].values)
+            parent_level = df_regions['Parent ID'][nan_loc].values
+            a, b = ismember(parent_level, df_regions['Structural ID'].values)
+            assert len(a) == len(b) == np.sum(nan_loc)
+            level = df_regions['level'].values[b] + 1
+            df_regions.loc[nan_loc, 'level'] = level
+
+        # lateralize
+        df_regions_left = df_regions.iloc[np.array(df_regions['Structural ID'] > 0), :].copy()
+        df_regions_left['Structural ID'] = - df_regions_left['Structural ID']
+        df_regions_left['Parent ID'] = - df_regions_left['Parent ID']
+        df_regions_left['Allen Full name'] = \
+            df_regions_left['Allen Full name'].apply(lambda x: x + ' (left)')
+        df_regions = pd.concat((df_regions, df_regions_left), axis=0)
+
+        # insert void
+        void = [{'Structural ID': int(0), 'Franklin-Paxinos Full Name': 'void', 'Franklin-Paxinos abbreviation': 'void',
+                'Parent ID': int(0), 'structure Order': 0, 'red': 0, 'green': 0, 'blue': 0, 'Allen Full name': 'void',
+                 'Allen abbreviation': 'void'}]
+        df_regions = pd.concat([pd.DataFrame(void), df_regions], ignore_index=True)
+
+        # converts colors to RGB uint8 array
+        c = np.c_[df_regions['red'], df_regions['green'], df_regions['blue']].astype(np.uint32)
+
+        super().__init__(id=df_regions['Structural ID'].to_numpy().astype(np.int64),
+                         name=df_regions['Allen Full name'].to_numpy(),
+                         acronym=df_regions['Allen abbreviation'].to_numpy(),
+                         rgb=c,
+                         level=df_regions['level'].to_numpy().astype(np.uint16),
+                         parent=df_regions['Parent ID'].to_numpy(),
+                         order=df_regions['structure Order'].to_numpy().astype(np.uint16))
+
+        self.n_lr = int((len(self.id) - 1) / 2)
+        self._compute_mappings()
+        self.default_mapping = 'FranklinPaxinos'
+
+    def _compute_mappings(self):
+        self.mappings = {
+            'FranklinPaxinos': self._mapping_from_regions_list(np.unique(np.abs(self.id)), lateralize=False),
+            'FranklinPaxinos-lr': np.arange(self.id.size),
+        }
+
+
+class BrainRegions(_BrainRegions):
+    """
+    ibllib.atlas.regions.BrainRegions(brainmap='Allen')
+    The Allen atlas ids are kept intact but lateralized as follows: labels are duplicated
+     and ids multiplied by -1, with the understanding that left hemisphere regions have negative
+     ids.
+    """
+    def __init__(self):
+        df_regions = pd.read_csv(ALLEN_FILE_REGIONS)
+        # lateralize
+        df_regions_left = df_regions.iloc[np.array(df_regions.id > 0), :].copy()
+        df_regions_left['id'] = - df_regions_left['id']
+        df_regions_left['parent_structure_id'] = - df_regions_left['parent_structure_id']
+        df_regions_left['name'] = df_regions_left['name'].apply(lambda x: x + ' (left)')
+        df_regions = pd.concat((df_regions, df_regions_left), axis=0)
+        # converts colors to RGB uint8 array
+        c = np.uint32(df_regions.color_hex_triplet.map(
+            lambda x: int(x, 16) if isinstance(x, str) else 256 ** 3 - 1))
+        c = np.flip(np.reshape(c.view(np.uint8), (df_regions.id.size, 4))[:, :3], 1)
+        c[0, :] = 0  # set the void region to black
+        # creates the BrainRegion instance
+        super().__init__(id=df_regions.id.to_numpy(),
+                         name=df_regions.name.to_numpy(),
+                         acronym=df_regions.acronym.to_numpy(),
+                         rgb=c,
+                         level=df_regions.depth.to_numpy().astype(np.uint16),
+                         parent=df_regions.parent_structure_id.to_numpy(),
+                         order=df_regions.graph_order.to_numpy().astype(np.uint16))
+        # mappings are indices not ids: they range from 0 to n regions -1
+        mappings = pd.read_parquet(FILE_MAPPINGS)
+        self.mappings = {k: mappings[k].to_numpy() for k in mappings}
+        self.n_lr = int((len(self.id) - 1) / 2)
+        self.default_mapping = 'Allen'
+
+    def _compute_mappings(self):
+        """
+        Recomputes the mapping indices for all mappings
+        This is left mainly as a reference for adding future mappings as this take a few seconds
+        to execute. In production,we use the MAPPING_FILES pqt to avoid recomputing at each \
+        instantiation
+        """
+        beryl = np.load(Path(__file__).parent.joinpath('beryl.npy'))
+        cosmos = np.load(Path(__file__).parent.joinpath('cosmos.npy'))
+        swanson = np.load(Path(__file__).parent.joinpath('swanson_regions.npy'))
+        self.mappings = {
+            'Allen': self._mapping_from_regions_list(np.unique(np.abs(self.id)), lateralize=False),
+            'Allen-lr': np.arange(self.id.size),
+            'Beryl': self._mapping_from_regions_list(beryl, lateralize=False),
+            'Beryl-lr': self._mapping_from_regions_list(beryl, lateralize=True),
+            'Cosmos': self._mapping_from_regions_list(cosmos, lateralize=False),
+            'Cosmos-lr': self._mapping_from_regions_list(cosmos, lateralize=True),
+            'Swanson': self._mapping_from_regions_list(swanson, lateralize=False),
+            'Swanson-lr': self._mapping_from_regions_list(swanson, lateralize=True),
+        }
+        pd.DataFrame(self.mappings).to_parquet(FILE_MAPPINGS)
+
+    def compute_hierarchy(self):
+        """
+        Creates a self.hierarchy attributes that is a n_levels by n_region array
+        of indices. This is useful to perform fast vectorized computations of
+        ancestors and descendants.
+        :return:
+        """
+        if hasattr(self, 'hierarchy'):
+            return
+        n_levels = np.max(self.level)
+        n_regions = self.id.size
+        # creates the parent index. Void and root are omitted from intersection
+        # as they figure as NaN
+        pmask, i_p = ismember(self.parent, self.id)
+        self.iparent = np.arange(n_regions)
+        self.iparent[pmask] = i_p
+        # the last level of the hierarchy is the actual mapping, then going up level per level
+        # we assign the parend index
+        self.hierarchy = np.tile(np.arange(n_regions), (n_levels, 1))
+        _mask = np.zeros(n_regions, bool)
+        for lev in np.flipud(np.arange(n_levels)):
+            if lev < (n_levels - 1):
+                self.hierarchy[lev, _mask] = self.iparent[self.hierarchy[lev + 1, _mask]]
+            sel = self.level == (lev + 1)
+            self.hierarchy[lev, sel] = np.where(sel)[0]
+            _mask[sel] = True
+
+    def remap(self, region_ids, source_map='Allen', target_map='Beryl'):
+        """
+        Remap atlas regions ids from source map to target map
+        :param region_ids: atlas ids to map
+        :param source_map: map name which original region_ids are in
+        :param target_map: map name onto which to map
+        :return:
+        """
+        _, inds = ismember(region_ids, self.id[self.mappings[source_map]])
+        return self.id[self.mappings[target_map][inds]]
+
+
 def regions_from_allen_csv():
     """
     Reads csv file containing the ALlen Ontology and instantiates a BrainRegions object
