@@ -1,4 +1,4 @@
-"""Base Extractor classes
+"""Base Extractor classes.
 A module for the base Extractor classes.  The Extractor, given a session path, will extract the
 processed data from raw hardware files and optionally save them.
 """
@@ -46,19 +46,32 @@ class BaseExtractor(abc.ABC):
         return out, files
 
     def _save(self, data, path_out=None):
-        # Chack if self.save_namesis of the same length of out
+        # Check if self.save_names is of the same length of out
         if not path_out:
             path_out = self.session_path.joinpath(self.default_path)
-        path_out.mkdir(exist_ok=True, parents=True)
 
         def _write_to_disk(file_path, data):
-            """Implements different save calls depending on file extension"""
+            """Implements different save calls depending on file extension.
+
+            Parameters
+            ----------
+            file_path : pathlib.Path
+                The location to save the data.
+            data : pandas.DataFrame, numpy.ndarray
+                The data to save
+
+            """
             csv_separators = {
                 ".csv": ",",
                 ".ssv": " ",
-                ".tsv": "\t",
+                ".tsv": "\t"
             }
+            # Ensure empty files are not created; we expect all datasets to have a non-zero size
+            if getattr(data, 'size', len(data)) == 0:
+                filename = file_path.relative_to(self.session_path).as_posix()
+                raise ValueError(f'Data for {filename} appears to be empty')
             file_path = Path(file_path)
+            file_path.parent.mkdir(exist_ok=True, parents=True)
             if file_path.suffix == ".npy":
                 np.save(file_path, data)
             elif file_path.suffix in [".parquet", ".pqt"]:
@@ -66,7 +79,7 @@ class BaseExtractor(abc.ABC):
                     _logger.error("Data is not a panda's DataFrame object")
                     raise TypeError("Data is not a panda's DataFrame object")
                 data.to_parquet(file_path)
-            elif file_path.suffix in [".csv", ".ssv", ".tsv"]:
+            elif file_path.suffix in csv_separators:
                 sep = csv_separators[file_path.suffix]
                 data.to_csv(file_path, sep=sep)
                 # np.savetxt(file_path, data, delimiter=sep)
@@ -106,8 +119,9 @@ class BaseBpodTrialsExtractor(BaseExtractor):
 
     bpod_trials = None
     settings = None
+    task_collection = None
 
-    def extract(self, bpod_trials=None, settings=None, **kwargs):
+    def extract(self, task_collection='raw_behavior_data', bpod_trials=None, settings=None, **kwargs):
         """
         :param: bpod_trials (optional) bpod trials from jsonable in a dictionary
         :param: settings (optional) bpod iblrig settings json file in a dictionary
@@ -118,10 +132,11 @@ class BaseBpodTrialsExtractor(BaseExtractor):
         """
         self.bpod_trials = bpod_trials
         self.settings = settings
+        self.task_collection = task_collection
         if self.bpod_trials is None:
-            self.bpod_trials = raw.load_data(self.session_path)
+            self.bpod_trials = raw.load_data(self.session_path, task_collection=self.task_collection)
         if not self.settings:
-            self.settings = raw.load_settings(self.session_path)
+            self.settings = raw.load_settings(self.session_path, task_collection=self.task_collection)
         if self.settings is None:
             self.settings = {"IBLRIG_VERSION_TAG": "100.0.0"}
         elif self.settings["IBLRIG_VERSION_TAG"] == "":
@@ -210,14 +225,14 @@ def get_task_extractor_type(task_name):
     return task_type
 
 
-def get_session_extractor_type(session_path):
+def get_session_extractor_type(session_path, task_collection='raw_behavior_data'):
     """
     From a session path, loads the settings file, finds the task and checks if extractors exist
     task names examples:
     :param session_path:
     :return: bool
     """
-    settings = load_settings(session_path)
+    settings = load_settings(session_path, task_collection=task_collection)
     if settings is None:
         _logger.error(f'ABORT: No data found in "raw_behavior_data" folder {session_path}')
         return False
@@ -230,7 +245,7 @@ def get_session_extractor_type(session_path):
 
 def get_pipeline(session_path):
     """
-    Get the pre-processinf pipeline name from a session path
+    Get the pre-processing pipeline name from a session path
     :param session_path:
     :return:
     """
@@ -248,5 +263,7 @@ def _get_pipeline_from_task_type(stype):
         return 'ephys'
     elif stype in ['habituation', 'training', 'biased', 'biased_opto']:
         return 'training'
+    elif 'widefield' in stype:
+        return 'widefield'
     else:
         return stype
