@@ -14,7 +14,7 @@ from ibllib.plots import squares, vertical_lines
 from ibllib.io.video import assert_valid_label, VideoStreamer
 from iblutil.numerical import within_ranges
 from ibllib.io.extractors.base import get_session_extractor_type
-from ibllib.io.extractors.ephys_fpga import get_sync_fronts, get_main_probe_sync
+from ibllib.io.extractors.ephys_fpga import get_sync_fronts, get_sync_and_chn_map
 import ibllib.io.raw_data_loaders as raw
 from ibllib.io.extractors.base import (
     BaseBpodTrialsExtractor,
@@ -23,7 +23,7 @@ from ibllib.io.extractors.base import (
     _get_task_types_json_config
 )
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 
 
 def extract_camera_sync(sync, chmap=None):
@@ -71,7 +71,7 @@ class CameraTimestampsFPGA(BaseExtractor):
         _logger.setLevel(self._log_level)
 
     def _extract(self, sync=None, chmap=None, video_path=None,
-                 display=False, extrapolate_missing=True):
+                 display=False, extrapolate_missing=True, **kwargs):
         """
         The raw timestamps are taken from the FPGA. These are the times of the camera's frame TTLs.
         If the pin state file exists, these timestamps are aligned to the video frames using the
@@ -141,6 +141,40 @@ class CameraTimestampsFPGA(BaseExtractor):
         return raw_ts
 
 
+class CameraTimestampsCamlog(BaseExtractor):
+    def __init__(self, label, session_path=None):
+        super().__init__(session_path)
+        self.label = assert_valid_label(label)
+        self.save_names = f'_ibl_{label}Camera.times.npy'
+        self.var_names = f'{label}_camera_timestamps'
+        self._log_level = _logger.level
+        _logger.setLevel(logging.DEBUG)
+
+    def __del__(self):
+        _logger.setLevel(self._log_level)
+
+    def _extract(self, sync=None, chmap=None, video_path=None,
+                 display=False, extrapolate_missing=True, **kwargs):
+
+        fpga_times = extract_camera_sync(sync=sync, chmap=chmap)
+        video_frames = get_video_length(self.session_path.joinpath('raw_video_data', f'_iblrig_{self.label}Camera.raw.mp4'))
+        raw_ts = fpga_times[self.label]
+
+        # For left camera sometimes we have one extra pulse than video frame
+        if (raw_ts.size - video_frames) == 1:
+            _logger.warning(f'One extra sync pulse detected for {self.label} camera')
+            raw_ts = raw_ts[:-1]
+        elif (raw_ts.size - video_frames) == -1:
+            _logger.warning(f'One extra video frame detected for {self.label} camera')
+            med_time = np.median(np.diff(raw_ts))
+            raw_ts = np.r_[raw_ts, np.array([raw_ts[-1] + med_time])]
+
+        assert video_frames == raw_ts.size, f'dimension mismatch between video frames and TTL pulses for {self.label} camera' \
+                                            f'by {np.abs(video_frames - raw_ts.size)} frames'
+
+        return raw_ts
+
+
 class CameraTimestampsBpod(BaseBpodTrialsExtractor):
     """
     Get the camera timestamps from the Bpod
@@ -159,7 +193,7 @@ class CameraTimestampsBpod(BaseBpodTrialsExtractor):
     def __del__(self):
         _logger.setLevel(self._log_level)
 
-    def _extract(self, video_path=None, display=False, extrapolate_missing=True):
+    def _extract(self, video_path=None, display=False, extrapolate_missing=True, **kwargs):
         """
         The raw timestamps are taken from the Bpod. These are the times of the camera's frame TTLs.
         If the pin state file exists, these timestamps are aligned to the video frames using the
@@ -188,8 +222,9 @@ class CameraTimestampsBpod(BaseBpodTrialsExtractor):
         # is empty, or contains only one value (i.e. doesn't change)
         if gpio is not None and gpio['indices'].size > 1:
             _logger.info('Aligning to audio TTLs')
+            task_collection = kwargs.get('task_collection', 'raw_behavior_data')
             # Extract audio TTLs
-            _, audio = raw.load_bpod_fronts(self.session_path, self.bpod_trials)
+            _, audio = raw.load_bpod_fronts(self.session_path, data=self.bpod_trials, task_collection=task_collection)
             _, ts = raw.load_camera_ssv_times(self.session_path, 'left')
             """
             There are many audio TTLs that are for some reason missed by the GPIO.  Conversely
@@ -642,28 +677,56 @@ def groom_pin_state(gpio, audio, ts, tolerance=2., display=False, take='first',
     return gpio, audio_, fcn_a2b(ts)
 
 
-def extract_all(session_path, session_type=None, save=True, **kwargs):
+def extract_all(session_path, sync_type=None, save=True, **kwargs):
     """
     For the IBL ephys task, reads ephys binary file and extract:
         -   video time stamps
-    :param session_path: '/path/to/subject/yyyy-mm-dd/001'
     :param session_type: the session type to extract, i.e. 'ephys', 'training' or 'biased'. If
     None the session type is inferred from the settings file.
     :param save: Bool, defaults to False
     :param kwargs: parameters to pass to the extractor
     :return: outputs, files
+
+    Parameters
+    ----------
+    session_path : str, pathlib.Path
+        The session path, e.g. '/path/to/subject/yyyy-mm-dd/001'.
+    sync_type : str
+        The sync label from the experiment description file.
+    sync_collection : str
+        The subdirectory containing the sync files.
+    save : bool
+        If True, save the camera timestamp files to disk.
+    session_type : str
+        (DEPRECATED) The session type, e.g. 'ephys'.
+    **kwargs
+        Extra keyword args to pass to the camera extractor classes.
+
+    Returns
+    -------
+    list of numpy.array
+        List of extracted output data, i.e. the camera times.
+    list of pathlib.Path
+        The paths of the extracted data, if save = True
     """
-    if session_type is None:
-        session_type = get_session_extractor_type(session_path)
-    if not session_type or session_type not in _get_task_types_json_config().values():
-        raise ValueError(f"Session type {session_type} has no matching extractor")
-    elif 'ephys' in session_type:  # assume ephys == FPGA
+
+    sync_collection = kwargs.get('sync_collection', 'raw_ephys_data')
+    camlog = kwargs.get('camlog', False)
+
+    if not sync_type:  # infer from session type
+        session_type = kwargs.get('session_type') or get_session_extractor_type(session_path)
+        if not session_type or session_type not in _get_task_types_json_config().values():
+            raise ValueError(f"Session type {session_type} has no matching extractor")
+        else:
+            sync_type = 'nidq' if session_type == 'ephys' else 'bpod'
+
+    if sync_type == 'nidq':
         labels = assert_valid_label(kwargs.pop('labels', ('left', 'right', 'body')))
         labels = (labels,) if isinstance(labels, str) else labels  # Ensure list/tuple
-        extractor = [partial(CameraTimestampsFPGA, label) for label in labels]
+        CamExtractor = CameraTimestampsCamlog if camlog else CameraTimestampsFPGA
+        extractor = [partial(CamExtractor, label) for label in labels]
         if 'sync' not in kwargs:
-            kwargs['sync'], kwargs['chmap'] = \
-                get_main_probe_sync(session_path, bin_exists=kwargs.pop('bin_exists', False))
+            kwargs['sync'], kwargs['chmap'] = get_sync_and_chn_map(session_path, sync_collection)
     else:  # assume Bpod otherwise
         assert kwargs.pop('labels', 'left'), 'only left camera is currently supported'
         extractor = CameraTimestampsBpod
