@@ -7,10 +7,11 @@ import logging
 from pathlib import Path
 from typing import Tuple
 
-import ibllib.io.raw_data_loaders as rawio
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+
+import ibllib.io.raw_data_loaders as rawio
 from ibllib.io.extractors import ephys_fpga
 from ibllib.io.extractors.base import BaseExtractor
 from ibllib.io.extractors.passive_plotting import (
@@ -58,7 +59,7 @@ min_dataset_types = [
 
 
 # load session fixtures
-def _load_passive_session_fixtures(session_path: str) -> dict:
+def _load_passive_session_fixtures(session_path: str, task_collection: str = 'raw_passive_data') -> dict:
     """load_passive_session_fixtures Loads corresponding ephys session fixtures
 
     :param session_path: the path to a session
@@ -66,8 +67,14 @@ def _load_passive_session_fixtures(session_path: str) -> dict:
     :return: position contrast phase delays and stim id's
     :rtype: dict
     """
-    settings = rawio.load_settings(session_path)
-    ses_nb = settings["SESSION_ORDER"][settings["SESSION_IDX"]]
+
+    # THIS CAN BE PREGENERATED SESSION NO
+    settings = rawio.load_settings(session_path, task_collection=task_collection)
+    ses_nb = settings['PREGENERATED_SESSION_NUM']
+    session_order = settings.get('SESSION_ORDER', None)
+    if session_order:  # TODO test this out and make sure it okay
+        assert settings["SESSION_ORDER"][settings["SESSION_IDX"]] == ses_nb
+
     path_fixtures = Path(ephys_fpga.__file__).parent.joinpath("ephys_sessions")
 
     fixture = {
@@ -79,7 +86,7 @@ def _load_passive_session_fixtures(session_path: str) -> dict:
     return fixture
 
 
-def _load_task_protocol(session_path: str) -> str:
+def _load_task_protocol(session_path: str, task_collection: str = 'raw_passive_data') -> str:
     """Find the IBL rig version used for the session
 
     :param session_path: the path to a session
@@ -87,7 +94,7 @@ def _load_task_protocol(session_path: str) -> str:
     :return: ibl rig task protocol version
     :rtype: str
     """
-    settings = rawio.load_settings(session_path)
+    settings = rawio.load_settings(session_path, task_collection=task_collection)
     ses_ver = settings["IBLRIG_VERSION_TAG"]
 
     return ses_ver
@@ -107,7 +114,7 @@ def _load_passive_stim_meta() -> dict:
 
 
 # 1/3 Define start and end times of the 3 passive periods
-def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet):
+def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet, thresh=3.0):
     """
     Find timestamps of spacer signal.
     :param spacer_template: list of indices where ttl signal changes
@@ -118,6 +125,8 @@ def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet):
     :type ttl_signal: array-like
     :param t_quiet: seconds between spacer and next stim
     :type t_quiet: float
+    :param thresh: threshold value for the fttl convolved signal (with spacer template) to pass over to detect a spacer
+    :type thresh: float
     :return: times of spacer onset/offset
     :rtype: n_spacer x 2 np.ndarray; first col onset times, second col offset
     """
@@ -132,7 +141,6 @@ def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet):
     # convolve cleaned diff ttl signal w/ spacer model
     conv_dttl = np.correlate(dttl, spacer_model, mode="full")
     # find spacer location
-    thresh = 3.0
     idxs_spacer_middle = np.where(
         (conv_dttl[1:-2] < thresh) & (conv_dttl[2:-1] > thresh) & (conv_dttl[3:] < thresh)
     )[0]
@@ -142,12 +150,12 @@ def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet):
     spacer_around = int((np.floor(len(spacer_model) / 2)))
     idxs_spacer_middle += 2 - spacer_around
 
-    # for each spacer make sure the times are monotonically increasing before
-    # and monotonically decreasing afterwards
+    # for each spacer make sure the times are monotonically non-decreasing before
+    # and monotonically non-increasing afterwards
     is_valid = np.zeros((idxs_spacer_middle.size), dtype=bool)
     for i, t in enumerate(idxs_spacer_middle):
-        before = all(np.diff(dttl[t - spacer_around:t]) > 0)
-        after = all(np.diff(dttl[t + 1:t + 1 + spacer_around]) < 0)
+        before = all(np.diff(dttl[t - spacer_around:t]) >= 0)
+        after = all(np.diff(dttl[t + 1:t + 1 + spacer_around]) <= 0)
         is_valid[i] = np.bitwise_and(before, after)
 
     idxs_spacer_middle = idxs_spacer_middle[is_valid]
@@ -163,36 +171,66 @@ def _get_spacer_times(spacer_template, jitter, ttl_signal, t_quiet):
     return spacer_times, conv_dttl
 
 
-def _get_passive_spacers(session_path, sync=None, sync_map=None):
+def _get_passive_spacers(session_path, sync_collection='raw_ephys_data',
+                         sync=None, sync_map=None, tmin=None, tmax=None):
     """
     load and get spacer information, do corr to find spacer timestamps
     returns t_passive_starts, t_starts, t_ends
     """
     if sync is None or sync_map is None:
-        sync, sync_map = ephys_fpga.get_main_probe_sync(session_path, bin_exists=False)
+        sync, sync_map = ephys_fpga.get_sync_and_chn_map(session_path, sync_collection=sync_collection)
     meta = _load_passive_stim_meta()
     # t_end_ephys = passive.ephysCW_end(session_path=session_path)
-    fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=None)
+    fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=tmin, tmax=tmax)
     fttl = ephys_fpga._clean_frame2ttl(fttl, display=False)
     spacer_template = (
         np.array(meta["VISUAL_STIM_0"]["ttl_frame_nums"], dtype=np.float32) / FRAME_FS
     )
     jitter = 3 / FRAME_FS  # allow for 3 screen refresh as jitter
     t_quiet = meta["VISUAL_STIM_0"]["delay_around"]
-    spacer_times, _ = _get_spacer_times(
+    spacer_times, conv_dttl = _get_spacer_times(
         spacer_template=spacer_template, jitter=jitter, ttl_signal=fttl["times"], t_quiet=t_quiet
     )
 
     # Check correct number of spacers found
-    n_exp_spacer = np.sum(np.array(meta["STIM_ORDER"]) == 0)  # Hardcoded 0 for spacer
+    n_exp_spacer = np.sum(np.array(meta['STIM_ORDER']) == 0)  # Hardcoded 0 for spacer
     if n_exp_spacer != np.size(spacer_times) / 2:
-        raise ValueError(
-            f"The number of expected spacer ({n_exp_spacer}) "
-            f"is different than the one found on the raw "
-            f"trace ({np.size(spacer_times)/2})"
-        )
+        error_nspacer = True
+        # sometimes the first spacer is truncated
+        # assess whether the first spacer is undetected, and then launch another spacer detection on truncated fttl
+        # with a lower threshold value
+        # Note: take *3 for some margin
+        if spacer_times[0][0] > (spacer_template[-1] + jitter) * 3 and (np.size(spacer_times) / 2) == n_exp_spacer - 1:
+            # Truncate signals
+            fttl_t = fttl["times"][np.where(fttl["times"] < spacer_times[0][0])]
+            conv_dttl_t = conv_dttl[np.where(fttl["times"] < spacer_times[0][0])]
+            ddttl = np.diff(np.diff(fttl_t))
+            # Find spacer location
+            # NB: cannot re-use the same algo for spacer detection as conv peaks towards spacer end
+            # 1. Find time point at which conv raises above a given threshold value
+            thresh = 2.0
+            idx_nearend_spacer = int(np.where((conv_dttl_t[1:-2] < thresh) & (conv_dttl_t[2:-1] > thresh))[0])
+            ddttl = ddttl[0:idx_nearend_spacer]
+            # 2. Find time point before this, for which fttl diff increase/decrease (this is the middle of spacer)
+            indx_middle = np.where((ddttl[0:-1] > 0) & (ddttl[1:] < 0))[0]
+            if len(indx_middle) == 1:
+                # 3. Add 1/2 spacer to middle idx to get the spacer end indx
+                spacer_around = int((np.floor(len(spacer_template) / 2)))
+                idx_end = int(indx_middle + spacer_around) + 1
+                spacer_times = np.insert(spacer_times, 0, np.array([fttl["times"][0], fttl["times"][idx_end]]), axis=0)
+                error_nspacer = False
 
-    spacer_times = np.r_[spacer_times.flatten(), sync["times"][-1]]
+        if error_nspacer:
+            raise ValueError(
+                f'The number of expected spacer ({n_exp_spacer}) '
+                f'is different than the one found on the raw '
+                f'trace ({int(np.size(spacer_times) / 2)})'
+            )
+
+    if tmax is None:
+        tmax = fttl['times'][-1]
+
+    spacer_times = np.r_[spacer_times.flatten(), tmax]
     return spacer_times[0], spacer_times[1::2], spacer_times[2::2]
 
 
@@ -267,10 +305,10 @@ def _reshape_RF(RF_file, meta_stim):
 
 
 # 3/3 Replay of task stimuli
-def _extract_passiveGabor_df(fttl: dict, session_path: str) -> pd.DataFrame:
+def _extract_passiveGabor_df(fttl: dict, session_path: str, task_collection: str = 'raw_passive_data') -> pd.DataFrame:
     # At this stage we want to define what pulses are and not quality control them.
-    # Pulses are stricty altternating with intevals
-    # find min max lengths for both (we don'tknow which are pulses and which are intervals yet)
+    # Pulses are strictly alternating with intervals
+    # find min max lengths for both (we don't know which are pulses and which are intervals yet)
     # trim edges of pulses
     diff0 = (np.min(np.diff(fttl["times"])[2:-2:2]), np.max(np.diff(fttl["times"])[2:-1:2]))
     diff1 = (np.min(np.diff(fttl["times"])[3:-2:2]), np.max(np.diff(fttl["times"])[3:-1:2]))
@@ -280,11 +318,9 @@ def _extract_passiveGabor_df(fttl: dict, session_path: str) -> pd.DataFrame:
     elif max(diff0 + diff1) in diff1:
         thresh = diff1[0]
     # Anything lower than the min length of intervals is a pulse
-    idx_start_stims = np.where((np.diff(fttl["times"]) < thresh) & (np.diff(fttl["times"]) > 0.1))[
-        0
-    ]
+    idx_start_stims = np.where((np.diff(fttl["times"]) < thresh) & (np.diff(fttl["times"]) > 0.1))[0]
     # Check if any pulse has been missed
-    # i.e. expected lenght (without first puls) and that it's alternating
+    # i.e. expected length (without first pulse) and that it's alternating
     if len(idx_start_stims) < NGABOR - 1 and np.any(np.diff(idx_start_stims) > 2):
         log.warning("Looks like one or more pulses were not detected, trying to extrapolate...")
         missing_where = np.where(np.diff(idx_start_stims) > 2)[0]
@@ -306,14 +342,14 @@ def _extract_passiveGabor_df(fttl: dict, session_path: str) -> pd.DataFrame:
     # intervals dstype requires reshaping of start and end times
     passiveGabor_intervals = np.array([(x, y) for x, y in zip(start_times, end_times)])
 
-    # Check length of presentation of stim is  within 150msof expected
+    # Check length of presentation of stim is within 150ms of expected
     if not np.allclose([y - x for x, y in passiveGabor_intervals], 0.3, atol=0.15):
         log.warning("Some Gabor presentation lengths seem wrong.")
 
     assert (
         len(passiveGabor_intervals) == NGABOR
     ), f"Wrong number of Gabor stimuli detected: {len(passiveGabor_intervals)} / {NGABOR}"
-    fixture = _load_passive_session_fixtures(session_path)
+    fixture = _load_passive_session_fixtures(session_path, task_collection)
     passiveGabor_properties = fixture["pcs"]
     passiveGabor_table = np.append(passiveGabor_intervals, passiveGabor_properties, axis=1)
     columns = ["start", "stop", "position", "contrast", "phase"]
@@ -416,14 +452,14 @@ def _extract_passiveAudio_intervals(audio: dict, rig_version: str) -> Tuple[np.a
 
 
 # ------------------------------------------------------------------
-def extract_passive_periods(
-    session_path: str, sync: dict = None, sync_map: dict = None
-) -> pd.DataFrame:
+def extract_passive_periods(session_path: str, sync_collection: str = 'raw_ephys_data', sync: dict = None,
+                            sync_map: dict = None, tmin=None, tmax=None) -> pd.DataFrame:
+
     if sync is None or sync_map is None:
-        sync, sync_map = ephys_fpga.get_main_probe_sync(session_path, bin_exists=False)
+        sync, sync_map = ephys_fpga.get_sync_and_chn_map(session_path, sync_collection)
 
     t_start_passive, t_starts, t_ends = _get_passive_spacers(
-        session_path, sync=sync, sync_map=sync_map
+        session_path, sync_collection, sync=sync, sync_map=sync_map, tmin=tmin, tmax=tmax
     )
     t_starts_col = np.insert(t_starts, 0, t_start_passive)
     t_ends_col = np.insert(t_ends, 0, t_ends[-1])
@@ -440,7 +476,8 @@ def extract_passive_periods(
 
 
 def extract_rfmapping(
-    session_path: str, sync: dict = None, sync_map: dict = None, trfm: np.array = None
+    session_path: str, sync_collection: str = 'raw_ephys_data', task_collection: str = 'raw_passive_data',
+        sync: dict = None, sync_map: dict = None, trfm: np.array = None
 ) -> Tuple[np.array, np.array]:
     meta = _load_passive_stim_meta()
     mkey = (
@@ -448,14 +485,14 @@ def extract_rfmapping(
         + {v: k for k, v in meta["VISUAL_STIMULI"].items()}["receptive_field_mapping"]
     )
     if sync is None or sync_map is None:
-        sync, sync_map = ephys_fpga.get_main_probe_sync(session_path, bin_exists=False)
+        sync, sync_map = ephys_fpga.get_sync_and_chn_map(session_path, sync_collection)
     if trfm is None:
-        passivePeriods_df = extract_passive_periods(session_path, sync=sync, sync_map=sync_map)
+        passivePeriods_df = extract_passive_periods(session_path, sync_collection, sync=sync, sync_map=sync_map)
         trfm = passivePeriods_df.RFM.values
 
     fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=trfm[0], tmax=trfm[1])
     fttl = ephys_fpga._clean_frame2ttl(fttl)
-    RF_file = Path().joinpath(session_path, "raw_passive_data", "_iblrig_RFMapStim.raw.bin")
+    RF_file = Path().joinpath(session_path, task_collection, "_iblrig_RFMapStim.raw.bin")
     passiveRFM_frames, RF_ttl_trace = _reshape_RF(RF_file=RF_file, meta_stim=meta[mkey])
     rf_id_up, rf_id_dw, RF_n_ttl_expected = _get_id_raisefall_from_analogttl(RF_ttl_trace)
     meta[mkey]["ttl_num"] = RF_n_ttl_expected
@@ -476,26 +513,28 @@ def extract_rfmapping(
 
 
 def extract_task_replay(
-    session_path: str, sync: dict = None, sync_map: dict = None, treplay: np.array = None
+    session_path: str, sync_collection: str = 'raw_ephys_data', task_collection: str = 'raw_passive_data',
+        sync: dict = None, sync_map: dict = None, treplay: np.array = None
 ) -> Tuple[pd.DataFrame, pd.DataFrame]:
+
     if sync is None or sync_map is None:
-        sync, sync_map = ephys_fpga.get_main_probe_sync(session_path, bin_exists=False)
+        sync, sync_map = ephys_fpga.get_sync_and_chn_map(session_path, sync_collection)
 
     if treplay is None:
-        passivePeriods_df = extract_passive_periods(session_path, sync=sync, sync_map=sync_map)
+        passivePeriods_df = extract_passive_periods(session_path, sync_collection, sync=sync, sync_map=sync_map)
         treplay = passivePeriods_df.taskReplay.values
 
-    fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=treplay[0])
+    # TODO need to check this is okay
+    fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=treplay[0], tmax=treplay[1])
     fttl = ephys_fpga._clean_frame2ttl(fttl)
-    passiveGabor_df = _extract_passiveGabor_df(fttl, session_path)
+    passiveGabor_df = _extract_passiveGabor_df(fttl, session_path, task_collection=task_collection)
 
-    bpod = ephys_fpga.get_sync_fronts(sync, sync_map["bpod"], tmin=treplay[0])
+    bpod = ephys_fpga.get_sync_fronts(sync, sync_map["bpod"], tmin=treplay[0], tmax=treplay[1])
     passiveValve_intervals = _extract_passiveValve_intervals(bpod)
 
-    task_version = _load_task_protocol(session_path)
-    audio = ephys_fpga.get_sync_fronts(sync, sync_map["audio"], tmin=treplay[0])
-    passiveTone_intervals, passiveNoise_intervals = _extract_passiveAudio_intervals(audio,
-                                                                                    task_version)
+    task_version = _load_task_protocol(session_path, task_collection)
+    audio = ephys_fpga.get_sync_fronts(sync, sync_map["audio"], tmin=treplay[0], tmax=treplay[1])
+    passiveTone_intervals, passiveNoise_intervals = _extract_passiveAudio_intervals(audio, task_version)
 
     passiveStims_df = np.concatenate(
         [passiveValve_intervals, passiveTone_intervals, passiveNoise_intervals], axis=1
@@ -510,6 +549,8 @@ def extract_task_replay(
 
 def extract_replay_debug(
     session_path: str,
+    sync_collection: str = 'raw_ephys_data',
+    task_collection: str = 'raw_passive_data',
     sync: dict = None,
     sync_map: dict = None,
     treplay: np.array = None,
@@ -517,10 +558,10 @@ def extract_replay_debug(
 ) -> Tuple[pd.DataFrame, pd.DataFrame]:
     # Load sessions sync channels, map
     if sync is None or sync_map is None:
-        sync, sync_map = ephys_fpga.get_main_probe_sync(session_path, bin_exists=False)
+        sync, sync_map = ephys_fpga.get_sync_and_chn_map(session_path, sync_collection)
 
     if treplay is None:
-        passivePeriods_df = extract_passive_periods(session_path, sync=sync, sync_map=sync_map)
+        passivePeriods_df = extract_passive_periods(session_path, sync_collection=sync_collection, sync=sync, sync_map=sync_map)
         treplay = passivePeriods_df.taskReplay.values
 
     if ax is None:
@@ -530,23 +571,22 @@ def extract_replay_debug(
     f.suptitle("/".join(str(session_path).split("/")[-5:]))
     plot_sync_channels(sync=sync, sync_map=sync_map, ax=ax)
 
-    passivePeriods_df = extract_passive_periods(session_path, sync=sync, sync_map=sync_map)
+    passivePeriods_df = extract_passive_periods(session_path, sync_collection=sync_collection, sync=sync, sync_map=sync_map)
     treplay = passivePeriods_df.taskReplay.values
 
     plot_passive_periods(passivePeriods_df, ax=ax)
 
     fttl = ephys_fpga.get_sync_fronts(sync, sync_map["frame2ttl"], tmin=treplay[0])
-    passiveGabor_df = _extract_passiveGabor_df(fttl, session_path)
+    passiveGabor_df = _extract_passiveGabor_df(fttl, session_path, task_collection=task_collection)
     plot_gabor_times(passiveGabor_df, ax=ax)
 
     bpod = ephys_fpga.get_sync_fronts(sync, sync_map["bpod"], tmin=treplay[0])
     passiveValve_intervals = _extract_passiveValve_intervals(bpod)
     plot_valve_times(passiveValve_intervals, ax=ax)
 
-    task_version = _load_task_protocol(session_path)
+    task_version = _load_task_protocol(session_path, task_collection)
     audio = ephys_fpga.get_sync_fronts(sync, sync_map["audio"], tmin=treplay[0])
-    passiveTone_intervals, passiveNoise_intervals = _extract_passiveAudio_intervals(audio,
-                                                                                    task_version)
+    passiveTone_intervals, passiveNoise_intervals = _extract_passiveAudio_intervals(audio, task_version)
     plot_audio_times(passiveTone_intervals, passiveNoise_intervals, ax=ax)
 
     passiveStims_df = np.concatenate(
@@ -576,32 +616,37 @@ class PassiveChoiceWorld(BaseExtractor):
         "passiveStims_df",
     )
 
-    def _extract(
-        self, sync: dict = None, sync_map: dict = None, plot: bool = False, **kwargs
-    ) -> tuple:
+    def _extract(self, sync_collection: str = 'raw_ephys_data', task_collection: str = 'raw_passive_data', sync: dict = None,
+                 sync_map: dict = None, plot: bool = False, **kwargs) -> tuple:
         if sync is None or sync_map is None:
-            sync, sync_map = ephys_fpga.get_main_probe_sync(self.session_path, bin_exists=False)
+            sync, sync_map = ephys_fpga.get_sync_and_chn_map(self.session_path, sync_collection)
+
+        # Get the start and end times of this protocol
+        if (protocol_number := kwargs.get('protocol_number')) is not None:  # look for spacer
+            # The spacers are TTLs generated by Bpod at the start of each protocol
+            bpod = ephys_fpga.get_sync_fronts(sync, sync_map['bpod'])
+            tmin, tmax = ephys_fpga.get_protocol_period(self.session_path, protocol_number, bpod)
+        else:
+            tmin = tmax = None
 
         # Passive periods
-        passivePeriods_df = extract_passive_periods(
-            self.session_path, sync=sync, sync_map=sync_map
-        )
+        passivePeriods_df = extract_passive_periods(self.session_path, sync_collection=sync_collection, sync=sync,
+                                                    sync_map=sync_map, tmin=tmin, tmax=tmax)
         trfm = passivePeriods_df.RFM.values
         treplay = passivePeriods_df.taskReplay.values
 
         try:
             # RFMapping
-            passiveRFM_times = extract_rfmapping(
-                self.session_path, sync=sync, sync_map=sync_map, trfm=trfm
-            )
+            passiveRFM_times = extract_rfmapping(self.session_path, sync_collection=sync_collection,
+                                                 task_collection=task_collection, sync=sync, sync_map=sync_map, trfm=trfm)
         except Exception as e:
             log.error(f"Failed to extract RFMapping datasets: {e}")
             passiveRFM_times = None
 
         try:
             (passiveGabor_df, passiveStims_df,) = extract_task_replay(
-                self.session_path, sync=sync, sync_map=sync_map, treplay=treplay
-            )
+                self.session_path, sync_collection=sync_collection, task_collection=task_collection, sync=sync,
+                sync_map=sync_map, treplay=treplay)
         except Exception as e:
             log.error(f"Failed to extract task replay stimuli: {e}")
             passiveGabor_df, passiveStims_df = (None, None)
