@@ -4,7 +4,7 @@ import numpy as np
 from ibllib.io.extractors import biased_trials
 from ibllib.io.extractors.base import BaseBpodTrialsExtractor
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 
 
 class LaserBool(BaseBpodTrialsExtractor):
