@@ -12,18 +12,19 @@ import logging
 import wave
 from collections import OrderedDict
 from datetime import datetime
-from pathlib import Path
+from pathlib import Path, PureWindowsPath
 from typing import Union
 
+from dateutil import parser as dateparser
 from pkg_resources import parse_version
 import numpy as np
 import pandas as pd
 
 from iblutil.io import jsonable
 from ibllib.io.video import assert_valid_label
-from ibllib.time import uncycle_pgts, convert_pgts
+from ibllib.time import uncycle_pgts, convert_pgts, date2isostr
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 
 
 def trial_times_to_times(raw_trial):
@@ -67,17 +68,18 @@ def trial_times_to_times(raw_trial):
     return raw_trial
 
 
-def load_bpod(session_path):
+def load_bpod(session_path, task_collection='raw_behavior_data'):
     """
     Load both settings and data from bpod (.json and .jsonable)
 
     :param session_path: Absolute path of session folder
+    :param task_collection: Collection within sesison path with behavior data
     :return: dict settings and list of dicts data
     """
-    return load_settings(session_path), load_data(session_path)
+    return load_settings(session_path, task_collection), load_data(session_path, task_collection)
 
 
-def load_data(session_path: Union[str, Path], time='absolute'):
+def load_data(session_path: Union[str, Path], task_collection='raw_behavior_data', time='absolute'):
     """
     Load PyBpod data files (.jsonable).
 
@@ -90,12 +92,12 @@ def load_data(session_path: Union[str, Path], time='absolute'):
     :rtype: list of dicts
     """
     if session_path is None:
-        _logger.warning("No data loaded: session_path is None")
+        _logger.warning('No data loaded: session_path is None')
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
-    path = next(path.glob("_iblrig_taskData.raw*.jsonable"), None)
+    path = Path(session_path).joinpath(task_collection)
+    path = next(path.glob('_iblrig_taskData.raw*.jsonable'), None)
     if not path:
-        _logger.warning("No data loaded: could not find raw data file")
+        _logger.warning('No data loaded: could not find raw data file')
         return None
     data = jsonable.read(path)
     if time == 'absolute':
@@ -300,7 +302,52 @@ def load_camera_gpio(session_path, label: str, as_dicts=False):
     return gpio
 
 
-def load_settings(session_path: Union[str, Path]):
+def _read_settings_json_compatibility_enforced(settings):
+    """
+    Patch iblrig settings for compatibility across rig versions.
+
+    Parameters
+    ----------
+    settings : pathlib.Path, dict
+        Either a _iblrig_taskSettings.raw.json file path or the loaded settings.
+
+    Returns
+    -------
+    dict
+        The task settings patched for compatibility.
+    """
+    if isinstance(settings, dict):
+        md = settings.copy()
+    else:
+        with open(settings) as js:
+            md = json.load(js)
+    if 'IS_MOCK' not in md.keys():
+        md['IS_MOCK'] = False
+    if not md.get('IBLRIG_VERSION_TAG'):
+        _logger.warning("You appear to be on an untagged version...")
+        return md
+    if 'IBLRIG_VERSION_TAG' not in md.keys():
+        md['IBLRIG_VERSION_TAG'] = ''
+    # 2018-12-05 Version 3.2.3 fixes (permanent fixes in IBL_RIG from 3.2.4 on)
+    if parse_version(md.get('IBLRIG_VERSION_TAG') or '3.2.3') <= parse_version('3.2.3'):
+        if 'LAST_TRIAL_DATA' in md.keys():
+            md.pop('LAST_TRIAL_DATA')
+        if 'weighings' in md['PYBPOD_SUBJECT_EXTRA'].keys():
+            md['PYBPOD_SUBJECT_EXTRA'].pop('weighings')
+        if 'water_administration' in md['PYBPOD_SUBJECT_EXTRA'].keys():
+            md['PYBPOD_SUBJECT_EXTRA'].pop('water_administration')
+        if 'IBLRIG_COMMIT_HASH' not in md.keys():
+            md['IBLRIG_COMMIT_HASH'] = 'f9d8905647dbafe1f9bdf78f73b286197ae2647b'
+        #  parse the date format to Django supported ISO
+        dt = dateparser.parse(md['SESSION_DATETIME'])
+        md['SESSION_DATETIME'] = date2isostr(dt)
+        # add the weight key if it doesn't already exist
+        if 'SUBJECT_WEIGHT' not in md.keys():
+            md['SUBJECT_WEIGHT'] = None
+    return md
+
+
+def load_settings(session_path: Union[str, Path], task_collection='raw_behavior_data'):
     """
     Load PyBpod Settings files (.json).
 
@@ -314,20 +361,17 @@ def load_settings(session_path: Union[str, Path]):
     if session_path is None:
         _logger.warning("No data loaded: session_path is None")
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_taskSettings.raw*.json"), None)
     if not path:
         _logger.warning("No data loaded: could not find raw settings file")
         return None
-    with open(path, 'r') as f:
-        settings = json.load(f)
-    if 'IBLRIG_VERSION_TAG' not in settings.keys():
-        settings['IBLRIG_VERSION_TAG'] = ''
+    settings = _read_settings_json_compatibility_enforced(path)
     return settings
 
 
-def load_stim_position_screen(session_path):
-    path = Path(session_path).joinpath("raw_behavior_data")
+def load_stim_position_screen(session_path, task_collection='raw_behavior_data'):
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_stimPositionScreen.raw*.csv"), None)
 
     data = pd.read_csv(path, sep=',', header=None, on_bad_lines='skip')
@@ -336,7 +380,7 @@ def load_stim_position_screen(session_path):
     return data
 
 
-def load_encoder_events(session_path, settings=False):
+def load_encoder_events(session_path, task_collection='raw_behavior_data', settings=False):
     """
     Load Rotary Encoder (RE) events raw data file.
 
@@ -361,10 +405,10 @@ def load_encoder_events(session_path, settings=False):
     """
     if session_path is None:
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_encoderEvents.raw*.ssv"), None)
     if not settings:
-        settings = load_settings(session_path)
+        settings = load_settings(session_path, task_collection=task_collection)
     if settings is None or settings['IBLRIG_VERSION_TAG'] == '':
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
         # auto-detect old files when version is not labeled
@@ -436,7 +480,7 @@ def _load_encoder_events_file_ge5(file_path):
     return _groom_wheel_data_ge5(data, label='_iblrig_encoderEvents.raw.ssv', path=file_path)
 
 
-def load_encoder_positions(session_path, settings=False):
+def load_encoder_positions(session_path, task_collection='raw_behavior_data', settings=False):
     """
     Load Rotary Encoder (RE) positions from raw data file within a session path.
 
@@ -465,10 +509,10 @@ def load_encoder_positions(session_path, settings=False):
     """
     if session_path is None:
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_encoderPositions.raw*.ssv"), None)
     if not settings:
-        settings = load_settings(session_path)
+        settings = load_settings(session_path, task_collection=task_collection)
     if settings is None or settings['IBLRIG_VERSION_TAG'] == '':
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
         # auto-detect old files when version is not labeled
@@ -485,7 +529,7 @@ def load_encoder_positions(session_path, settings=False):
         return _load_encoder_positions_file_lt5(path)
 
 
-def load_encoder_trial_info(session_path):
+def load_encoder_trial_info(session_path, task_collection='raw_behavior_data'):
     """
     Load Rotary Encoder trial info from raw data file.
 
@@ -516,7 +560,7 @@ def load_encoder_trial_info(session_path):
     """
     if session_path is None:
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_encoderTrialInfo.raw*.ssv"), None)
     if not path:
         return None
@@ -528,7 +572,7 @@ def load_encoder_trial_info(session_path):
     return data
 
 
-def load_ambient_sensor(session_path):
+def load_ambient_sensor(session_path, task_collection='raw_behavior_data'):
     """
     Load Ambient Sensor data from session.
 
@@ -546,7 +590,7 @@ def load_ambient_sensor(session_path):
     """
     if session_path is None:
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_ambientSensorData.raw*.jsonable"), None)
     if not path:
         return None
@@ -557,18 +601,18 @@ def load_ambient_sensor(session_path):
     return data
 
 
-def load_mic(session_path):
+def load_mic(session_path, task_collection='raw_behavior_data'):
     """
     Load Microphone wav file to np.array of len nSamples
 
-    :param session_path: Absoulte path of session folder
+    :param session_path: Absolute path of session folder
     :type session_path: str
     :return: An array of values of the sound waveform
     :rtype: numpy.array
     """
     if session_path is None:
         return
-    path = Path(session_path).joinpath("raw_behavior_data")
+    path = Path(session_path).joinpath(task_collection)
     path = next(path.glob("_iblrig_micData.raw*.wav"), None)
     if not path:
         return None
@@ -714,7 +758,7 @@ def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, ma
         return t0[ind0], t1[ind1]
 
 
-def load_bpod_fronts(session_path: str, data: list = False) -> list:
+def load_bpod_fronts(session_path: str, data: list = False, task_collection: str = 'raw_behavior_data') -> list:
     """load_bpod_fronts
     Loads BNC1 and BNC2 bpod channels times and polarities from session_path
 
@@ -726,7 +770,7 @@ def load_bpod_fronts(session_path: str, data: list = False) -> list:
     :rtype: list
     """
     if not data:
-        data = load_data(session_path)
+        data = load_data(session_path, task_collection)
 
     BNC1_fronts = np.array([[np.nan, np.nan]])
     BNC2_fronts = np.array([[np.nan, np.nan]])
@@ -804,3 +848,113 @@ def get_port_events(trial: dict, name: str = '') -> list:
     out = sorted(out)
 
     return out
+
+
+def load_widefield_mmap(session_path, dtype=np.uint16, shape=(540, 640), n_frames=None, mode='r'):
+    """
+    TODO Document this function
+
+    Parameters
+    ----------
+    session_path
+
+    Returns
+    -------
+
+    """
+    filepath = Path(session_path).joinpath('raw_widefield_data').glob('widefield.raw.*.dat')
+    filepath = next(filepath, None)
+    if not filepath:
+        _logger.warning("No data loaded: could not find raw data file")
+        return None
+
+    if type(dtype) is str:
+        dtype = np.dtype(dtype)
+
+    if n_frames is None:
+        # Get the number of samples from the file size
+        n_frames = int(filepath.stat().st_size / (np.prod(shape) * dtype.itemsize))
+
+    return np.memmap(str(filepath), mode=mode, dtype=dtype, shape=(int(n_frames), *shape))
+
+
+def patch_settings(session_path, collection='raw_behavior_data',
+                   new_collection=None, subject=None, number=None, date=None):
+    """Modify various details in a settings file.
+
+    This function makes it easier to change things like subject name in a settings as it will
+    modify the subject name in the myriad paths. NB: This saves the settings into the same location
+    it was loaded from.
+
+    Parameters
+    ----------
+    session_path : str, pathlib.Path
+        The session path containing the settings file.
+    collection : str
+        The subfolder containing the settings file.
+    new_collection : str
+        An optional new subfolder to change in the settings paths.
+    subject : str
+        An optional new subject name to change in the settings.
+    number : str, int
+        An optional new number to change in the settings.
+    date : str, datetime.date
+        An optional date to change in the settings.
+
+    Returns
+    -------
+    dict
+        The modified settings.
+    """
+    settings = load_settings(session_path, collection)
+    if not settings:
+        raise IOError('Settings file not found')
+
+    filename = PureWindowsPath(settings['SETTINGS_FILE_PATH']).name
+    file_path = Path(session_path).joinpath(collection, filename)
+
+    if subject:
+        # Patch subject name
+        old_subject = settings['SUBJECT_NAME']
+        settings['SUBJECT_NAME'] = subject
+        for k in settings.keys():
+            if isinstance(settings[k], str):
+                settings[k] = settings[k].replace(f'\\Subjects\\{old_subject}', f'\\Subjects\\{subject}')
+        settings['SESSION_NAME'] = '\\'.join([subject, *settings['SESSION_NAME'].split('\\')[1:]])
+        settings.pop('PYBPOD_SUBJECT_EXTRA', None)  # Get rid of Alyx subject info
+
+    if date:
+        # Patch session datetime
+        date = str(date)
+        old_date = settings['SESSION_DATE']
+        settings['SESSION_DATE'] = date
+        for k in settings.keys():
+            if isinstance(settings[k], str):
+                settings[k] = settings[k].replace(
+                    f'\\{settings["SUBJECT_NAME"]}\\{old_date}',
+                    f'\\{settings["SUBJECT_NAME"]}\\{date}'
+                )
+        settings['SESSION_DATETIME'] = date + settings['SESSION_DATETIME'][10:]
+
+    if number:
+        # Patch session number
+        old_number = settings['SESSION_NUMBER']
+        if isinstance(number, int):
+            number = f'{number:03}'
+        settings['SESSION_NUMBER'] = number
+        for k in settings.keys():
+            if isinstance(settings[k], str):
+                settings[k] = settings[k].replace(
+                    f'\\{settings["SESSION_DATE"]}\\{old_number}',
+                    f'\\{settings["SESSION_DATE"]}\\{number}'
+                )
+
+    if new_collection:
+        old_path = settings['SESSION_RAW_DATA_FOLDER']
+        new_path = PureWindowsPath(settings['SESSION_RAW_DATA_FOLDER']).with_name(new_collection)
+        for k in settings.keys():
+            if isinstance(settings[k], str):
+                settings[k] = settings[k].replace(old_path, str(new_path))
+    with open(file_path, 'w') as fp:
+        json.dump(settings, fp, indent=' ')
+    return settings
