@@ -20,7 +20,7 @@ from ibllib.ephys import ephysqc, spikes, sync_probes
 from ibllib.io import ffmpeg
 from ibllib.io.video import label_from_path, assert_valid_label
 from ibllib.io.extractors import ephys_fpga, ephys_passive, camera
-from ibllib.pipes import tasks
+from ibllib.pipes import tasks, base_tasks
 from ibllib.pipes.training_preprocessing import TrainingRegisterRaw as EphysRegisterRaw
 from ibllib.pipes.training_preprocessing import TrainingStatus as EphysTrainingStatus
 from ibllib.pipes.misc import create_alyx_probe_insertions
@@ -216,10 +216,12 @@ class SpikeSorting(tasks.Task):
     Pykilosort 2.5 pipeline
     """
     gpu = 1
-    io_charge = 70  # this jobs reads raw ap files
+    io_charge = 100  # this jobs reads raw ap files
     priority = 60
     level = 1  # this job doesn't depend on anything
     force = True
+    job_size = 'large'
+
     SHELL_SCRIPT = Path.home().joinpath(
         "Documents/PYTHON/iblscripts/deploy/serverpc/kilosort2/run_pykilosort.sh"
     )
@@ -269,10 +271,16 @@ class SpikeSorting(tasks.Task):
 
     @staticmethod
     def _fetch_pykilosort_run_version(log_file):
+        """
+        Parse the following line (2 formats depending on version) from the log files to get the version
+        '\x1b[0m15:39:37.919 [I] ibl:90               Starting Pykilosort version ibl_1.2.1, output in gnagga^[[0m\n'
+        '\x1b[0m15:39:37.919 [I] ibl:90               Starting Pykilosort version ibl_1.3.0^[[0m\n'
+        """
         with open(log_file) as fid:
             line = fid.readline()
-        version = re.search('version (.*), output', line).group(1)
-
+        version = re.search('version (.*), output', line)
+        version = version or re.search('version (.*)', line)  # old versions have output, new have a version line
+        version = re.sub('\\^[[0-9]+m', '', version.group(1))  # removes the coloring tags
         return f"pykilosort_{version}"
 
     @staticmethod
@@ -290,6 +298,35 @@ class SpikeSorting(tasks.Task):
             return ""
         return info.decode("utf-8").strip()
 
+    @staticmethod
+    def parse_version(v) -> packaging.version.Version:
+        """
+        Extracts and parses semantic version (major.minor.patch) from a version string.
+
+        Parameters
+        ----------
+        v : str
+            A version string containing a semantic version.
+
+        Returns
+        -------
+        packaging.version.Version
+            The parsed semantic version number
+
+        Examples
+        --------
+        >>> SpikeSorting.parse_version('ibl_1.2')
+        <Version('1.2')>
+        >>> SpikeSorting.parse_version('pykilosort_ibl_1.2.0-new')
+        <Version('1.2.0')>
+        >>> SpikeSorting.parse_version('ibl_0.2') < SpikeSorting.parse_version('pykilosort_v1')
+        True
+        """
+        m = re.search(r'(\d+\.?){1,3}', v)
+        if not m:
+            raise packaging.version.InvalidVersion(f'Failed to detect SemVer in "{v}"')
+        return packaging.version.parse(m.group())
+
     def setUp(self, probes=None):
         """
         Overwrite setup method to allow inputs and outputs to be only one probe
@@ -318,7 +355,7 @@ class SpikeSorting(tasks.Task):
             log_file = sorter_dir.joinpath(f"spike_sorting_{self.SPIKE_SORTER_NAME}.log")
             if log_file.exists():
                 run_version = self._fetch_pykilosort_run_version(log_file)
-                if packaging.version.parse(run_version) > packaging.version.parse('pykilosort_ibl_1.1.0'):
+                if self.parse_version(run_version) > self.parse_version('pykilosort_ibl_1.1.0'):
                     _logger.info(f"Already ran: spike_sorting_{self.SPIKE_SORTER_NAME}.log"
                                  f" found in {sorter_dir}, skipping.")
                     return sorter_dir
@@ -440,7 +477,7 @@ class SpikeSorting(tasks.Task):
                             out = get_aligned_channels(ins[0], chns, one=self.one, save_dir=probe_out_path)
                             out_files.extend(out)
 
-            except BaseException:
+            except Exception:
                 _logger.error(traceback.format_exc())
                 self.status = -1
                 continue
@@ -492,6 +529,9 @@ class EphysVideoCompress(tasks.Task):
     priority = 90
     level = 0
     force = False
+    job_size = 'large'
+    io_charge = 100
+
     signature = {
         'input_files': [('_iblrig_*Camera.raw.*', 'raw_video_data', True)],
         'output_files': [('_iblrig_*Camera.raw.mp4', 'raw_video_data', True)]
@@ -535,6 +575,7 @@ class EphysVideoSyncQc(tasks.Task):
     force = True
     signature = {
         'input_files': [('_iblrig_*Camera.raw.mp4', 'raw_video_data', True),
+                        ('_iblrig_*Camera.timestamps.ssv', 'raw_video_data', False),
                         ('_iblrig_*Camera.timestamps.npy', 'raw_video_data', False),
                         ('_iblrig_*Camera.frameData.bin', 'raw_video_data', False),
                         ('_iblrig_*Camera.GPIO.bin', 'raw_video_data', False),
@@ -619,22 +660,10 @@ class EphysTrials(tasks.Task):
                         ('_iblrig_encoderPositions.raw*', 'raw_behavior_data', True),
                         ('*wiring.json', 'raw_ephys_data*', False),
                         ('*.meta', 'raw_ephys_data*', True)],
-        'output_files': [('*trials.choice.npy', 'alf', True),
-                         ('*trials.contrastLeft.npy', 'alf', True),
-                         ('*trials.contrastRight.npy', 'alf', True),
-                         ('*trials.feedbackType.npy', 'alf', True),
-                         ('*trials.feedback_times.npy', 'alf', True),
-                         ('*trials.firstMovement_times.npy', 'alf', True),
+        'output_files': [('*trials.table.pqt', 'alf', True),
                          ('*trials.goCueTrigger_times.npy', 'alf', True),
-                         ('*trials.goCue_times.npy', 'alf', True),
-                         ('*trials.intervals.npy', 'alf', True),
                          ('*trials.intervals_bpod.npy', 'alf', True),
-                         ('*trials.itiDuration.npy', 'alf', False),
-                         ('*trials.probabilityLeft.npy', 'alf', True),
-                         ('*trials.response_times.npy', 'alf', True),
-                         ('*trials.rewardVolume.npy', 'alf', True),
                          ('*trials.stimOff_times.npy', 'alf', True),
-                         ('*trials.stimOn_times.npy', 'alf', True),
                          ('*wheel.position.npy', 'alf', True),
                          ('*wheel.timestamps.npy', 'alf', True),
                          ('*wheelMoves.intervals.npy', 'alf', True),
@@ -657,9 +686,14 @@ class EphysTrials(tasks.Task):
             "sessions", eid, "extended_qc", {"behavior": int(good_enough)}
         )
 
-    def _run(self, plot_qc=True):
+    def _extract_behaviour(self):
         dsets, out_files = ephys_fpga.extract_all(self.session_path, save=True)
 
+        return dsets, out_files
+
+    def _run(self, plot_qc=True):
+        dsets, out_files = self._extract_behaviour()
+
         if not self.one or self.one.offline:
             return out_files
 
@@ -680,8 +714,7 @@ class EphysTrials(tasks.Task):
                 plot_task = BehaviourPlots(session_id, self.session_path, one=self.one)
                 _ = plot_task.run()
                 self.plot_tasks.append(plot_task)
-
-            except BaseException:
+            except Exception:
                 _logger.error('Could not create Trials QC Plot')
                 _logger.error(traceback.format_exc())
                 self.status = -1
@@ -910,9 +943,10 @@ class EphysDLC(tasks.Task):
     """
     gpu = 1
     cpu = 4
-    io_charge = 90
+    io_charge = 100
     level = 2
     force = True
+    job_size = 'large'
 
     dlcenv = Path.home().joinpath('Documents', 'PYTHON', 'envs', 'dlcenv', 'bin', 'activate')
     scripts = Path.home().joinpath('Documents', 'PYTHON', 'iblscripts', 'deploy', 'serverpc', 'dlc')
@@ -1079,9 +1113,9 @@ class EphysPostDLC(tasks.Task):
                                  ('_iblrig_bodyCamera.raw.mp4', 'raw_video_data', True),
                                  ('_iblrig_leftCamera.raw.mp4', 'raw_video_data', True),
                                  ('_iblrig_rightCamera.raw.mp4', 'raw_video_data', True),
-                                 ('rightROIMotionEnergy.position.npy', 'alf', True),
-                                 ('leftROIMotionEnergy.position.npy', 'alf', True),
-                                 ('bodyROIMotionEnergy.position.npy', 'alf', True),
+                                 ('rightROIMotionEnergy.position.npy', 'alf', False),
+                                 ('leftROIMotionEnergy.position.npy', 'alf', False),
+                                 ('bodyROIMotionEnergy.position.npy', 'alf', False),
                                  ('_ibl_trials.table.pqt', 'alf', True),
                                  ('_ibl_wheel.position.npy', 'alf', True),
                                  ('_ibl_wheel.timestamps.npy', 'alf', True),
@@ -1271,6 +1305,7 @@ class EphysExtractionPipeline(tasks.Pipeline):
         tasks = OrderedDict()
         self.session_path = session_path
         # level 0
+        tasks['ExperimentDescriptionRegisterRaw'] = base_tasks.ExperimentDescriptionRegisterRaw(self.session_path)
         tasks["EphysRegisterRaw"] = EphysRegisterRaw(self.session_path)
         tasks["EphysPulses"] = EphysPulses(self.session_path)
         tasks["EphysRawQC"] = RawEphysQC(self.session_path)
