@@ -12,19 +12,23 @@ from graphviz import Digraph
 
 import ibllib
 from ibllib.oneibl import data_handlers
+from ibllib.oneibl.data_handlers import get_local_data_repository
+from ibllib.oneibl.registration import get_lab
+from iblutil.util import Bunch
 import one.params
 from one.api import ONE
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
+TASK_STATUS_SET = {'Waiting', 'Held', 'Started', 'Errored', 'Empty', 'Complete', 'Incomplete', 'Abandoned'}
 
 
 class Task(abc.ABC):
-    log = ""  # place holder to keep the log of the task for registratoin
+    log = ''  # place holder to keep the log of the task for registration
     cpu = 1   # CPU resource
     gpu = 0   # GPU resources: as of now, either 0 or 1
     io_charge = 5  # integer percentage
     priority = 30  # integer percentage, 100 means highest priority
-    ram = 4  # RAM needed to run (Go)
+    ram = 4  # RAM needed to run (GB)
     one = None  # one instance (optional)
     level = 0  # level in the pipeline hierarchy: level 0 means there is no parent task
     outputs = None  # place holder for a list of Path containing output files
@@ -33,9 +37,10 @@ class Task(abc.ABC):
     version = ibllib.__version__
     signature = {'input_files': [], 'output_files': []}  # list of tuples (filename, collection, required_flag)
     force = False  # whether or not to re-download missing input files on local server if not present
+    job_size = 'small'  # either 'small' or 'large', defines whether task should be run as part of the large or small job services
 
     def __init__(self, session_path, parents=None, taskid=None, one=None,
-                 machine=None, clobber=True, location='server'):
+                 machine=None, clobber=True, location='server', **kwargs):
         """
         Base task class
         :param session_path: session path
@@ -47,6 +52,7 @@ class Task(abc.ABC):
         :param location: location where task is run. Options are 'server' (lab local servers'), 'remote' (remote compute node,
         data required for task downloaded via one), 'AWS' (remote compute node, data required for task downloaded via AWS),
         or 'SDSC' (SDSC flatiron compute node) # TODO 'Globus' (remote compute node, data required for task downloaded via Globus)
+        :param args: running arguments
         """
         self.taskid = taskid
         self.one = one
@@ -54,12 +60,14 @@ class Task(abc.ABC):
         self.register_kwargs = {}
         if parents:
             self.parents = parents
+            self.level = max(p.level for p in self.parents) + 1
         else:
             self.parents = []
         self.machine = machine
         self.clobber = clobber
         self.location = location
         self.plot_tasks = []  # Plotting task/ tasks to create plot outputs during the task
+        self.kwargs = kwargs
 
     @property
     def name(self):
@@ -78,7 +86,7 @@ class Task(abc.ABC):
             -2: Didn't run as a lock was encountered
             -3: Incomplete
         """
-        # if taskid of one properties are not available, local run only without alyx
+        # if task id of one properties are not available, local run only without alyx
         use_alyx = self.one is not None and self.taskid is not None
         if use_alyx:
             # check that alyx user is logged in
@@ -95,17 +103,17 @@ class Task(abc.ABC):
         ch = logging.StreamHandler(log_capture_string)
         str_format = '%(asctime)s,%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s'
         ch.setFormatter(logging.Formatter(str_format))
-        _logger.addHandler(ch)
-        _logger.setLevel(logging.INFO)
-        _logger.info(f"Starting job {self.__class__}")
+        _logger.parent.addHandler(ch)
+        _logger.parent.setLevel(logging.INFO)
+        _logger.info(f'Starting job {self.__class__}')
         if self.machine:
-            _logger.info(f"Running on machine: {self.machine}")
-        _logger.info(f"running ibllib version {ibllib.__version__}")
+            _logger.info(f'Running on machine: {self.machine}')
+        _logger.info(f'running ibllib version {ibllib.__version__}')
         # setup
         start_time = time.time()
         try:
             setup = self.setUp(**kwargs)
-            _logger.info(f"Setup value is: {setup}")
+            _logger.info(f'Setup value is: {setup}')
             self.status = 0
             if not setup:
                 # case where outputs are present but don't have input files locally to rerun task
@@ -116,17 +124,17 @@ class Task(abc.ABC):
                 if self.gpu >= 1:
                     if not self._creates_lock():
                         self.status = -2
-                        _logger.info(f"Job {self.__class__} exited as a lock was found")
+                        _logger.info(f'Job {self.__class__} exited as a lock was found')
                         new_log = log_capture_string.getvalue()
                         self.log = new_log if self.clobber else self.log + new_log
-                        log_capture_string.close()
                         _logger.removeHandler(ch)
+                        ch.close()
                         return self.status
                 self.outputs = self._run(**kwargs)
-                _logger.info(f"Job {self.__class__} complete")
+                _logger.info(f'Job {self.__class__} complete')
         except Exception:
             _logger.error(traceback.format_exc())
-            _logger.info(f"Job {self.__class__} errored")
+            _logger.info(f'Job {self.__class__} errored')
             self.status = -1
 
         self.time_elapsed_secs = time.time() - start_time
@@ -138,13 +146,13 @@ class Task(abc.ABC):
         else:
             nout = 1
 
-        _logger.info(f"N outputs: {nout}")
-        _logger.info(f"--- {self.time_elapsed_secs} seconds run-time ---")
+        _logger.info(f'N outputs: {nout}')
+        _logger.info(f'--- {self.time_elapsed_secs} seconds run-time ---')
         # after the run, capture the log output, amend to any existing logs if not overwrite
         new_log = log_capture_string.getvalue()
         self.log = new_log if self.clobber else self.log + new_log
-        log_capture_string.close()
         _logger.removeHandler(ch)
+        ch.close()
         _logger.setLevel(logger_level)
         # tear down
         self.tearDown()
@@ -266,7 +274,7 @@ class Task(abc.ABC):
 
         if not everything_is_fine:
             for out in self.outputs:
-                _logger.error(f"{out}")
+                _logger.error(f'{out}')
             if raise_error:
                 raise FileNotFoundError("Missing outputs after task completion")
 
@@ -282,7 +290,7 @@ class Task(abc.ABC):
         everything_is_fine, files = self.assert_expected(self.input_files)
 
         if not everything_is_fine and raise_error:
-            raise FileNotFoundError("Missing inputs to run task")
+            raise FileNotFoundError('Missing inputs to run task')
 
         return everything_is_fine, files
 
@@ -294,7 +302,7 @@ class Task(abc.ABC):
             if len(actual_files) == 0 and expected_file[2]:
                 everything_is_fine = False
                 if not silent:
-                    _logger.error(f"Signature file expected {expected_file} not found")
+                    _logger.error(f'Signature file expected {expected_file} not found')
             else:
                 if len(actual_files) != 0:
                     files.append(actual_files[0])
@@ -306,7 +314,7 @@ class Task(abc.ABC):
         Gets the relevant data handler based on location argument
         :return:
         """
-        location = location or self.location
+        location = str.lower(location or self.location)
         if location == 'local':
             return data_handlers.LocalDataHandler(self.session_path, self.signature, one=self.one)
         self.one = self.one or ONE()
@@ -316,14 +324,16 @@ class Task(abc.ABC):
             dhandler = data_handlers.ServerGlobusDataHandler(self.session_path, self.signature, one=self.one)
         elif location == 'remote':
             dhandler = data_handlers.RemoteHttpDataHandler(self.session_path, self.signature, one=self.one)
-        elif location == 'AWS':
+        elif location == 'aws':
             dhandler = data_handlers.RemoteAwsDataHandler(self, self.session_path, self.signature, one=self.one)
-        elif location == 'SDSC':
+        elif location == 'sdsc':
             dhandler = data_handlers.SDSCDataHandler(self, self.session_path, self.signature, one=self.one)
+        else:
+            raise ValueError(f'Unknown location "{location}"')
         return dhandler
 
     @staticmethod
-    def make_lock_file(taskname="", time_out_secs=7200):
+    def make_lock_file(taskname='', time_out_secs=7200):
         """Creates a GPU lock file with a timeout of"""
         d = {'start': time.time(), 'name': taskname, 'time_out_secs': time_out_secs}
         with open(Task._lock_file_path(), 'w+') as fid:
@@ -377,6 +387,11 @@ class Pipeline(abc.ABC):
         if one and one.alyx.cache_mode and one.alyx.default_expiry.seconds > 1:
             _logger.warning('Alyx client REST cache active; this may cause issues with jobs')
         self.eid = eid
+        if self.one and not self.one.offline:
+            self.data_repo = get_local_data_repository(self.one.alyx)
+        else:
+            self.data_repo = None
+
         if session_path:
             self.session_path = session_path
             if not self.eid:
@@ -384,6 +399,21 @@ class Pipeline(abc.ABC):
                 self.eid = one.path2eid(session_path, query_type='remote') if self.one else None
         self.label = self.__module__ + '.' + type(self).__name__
 
+    @staticmethod
+    def _get_exec_name(obj):
+        """
+        For a class, get the executable name as it should be stored in Alyx. When the class
+        is created dynamically using the type() built-in function, need to revert to the base
+        class to be able to re-instantiate the class from the alyx dictionary on the client side
+        :param obj:
+        :return: string containing the full module plus class name
+        """
+        if obj.__module__ == 'abc':
+            exec_name = f'{obj.__class__.__base__.__module__}.{obj.__class__.__base__.__name__}'
+        else:
+            exec_name = f'{obj.__module__}.{obj.name}'
+        return exec_name
+
     def make_graph(self, out_dir=None, show=True):
         if not out_dir:
             out_dir = self.one.alyx.cache_dir if self.one else one.params.get().CACHE_DIR
@@ -409,51 +439,109 @@ class Pipeline(abc.ABC):
             m.view()
         return m
 
-    def create_alyx_tasks(self, rerun__status__in=None):
+    def create_alyx_tasks(self, rerun__status__in=None, tasks_list=None):
         """
         Instantiate the pipeline and create the tasks in Alyx, then create the jobs for the session
         If the jobs already exist, they are left untouched. The re-run parameter will re-init the
-        job by emptying the log and set the status to Waiting
-        :param rerun__status__in: by default no re-run. To re-run tasks if they already exist,
-        specify a list of statuses string that will be re-run, those are the possible choices:
-        ['Waiting', 'Started', 'Errored', 'Empty', 'Complete']
-        to always patch, the string '__all__' can also be provided
-        :return: list of alyx tasks dictionaries (existing and or created)
+        job by emptying the log and set the status to Waiting.
+
+        Parameters
+        ----------
+        rerun__status__in : list, str
+            To re-run tasks if they already exist, specify one or more statuses strings to will be
+            re-run, or '__all__' to re-run all tasks.
+        tasks_list : list
+            The list of tasks to create on Alyx. If None, uses self.tasks.
+
+        Returns
+        -------
+        list
+            List of Alyx task dictionaries (existing and/or created).
         """
-        rerun__status__in = rerun__status__in or []
-        if rerun__status__in == '__all__':
-            rerun__status__in = ['Waiting', 'Started', 'Errored', 'Empty', 'Complete']
+        rerun__status__in = ([rerun__status__in]
+                             if isinstance(rerun__status__in, str)
+                             else rerun__status__in or [])
+        if '__all__' in rerun__status__in:
+            rerun__status__in = [x for x in TASK_STATUS_SET if x != 'Abandoned']
         assert self.eid
         if self.one is None:
-            _logger.warning("No ONE instance found for Alyx connection, set the one property")
+            _logger.warning('No ONE instance found for Alyx connection, set the one property')
             return
-        tasks_alyx_pre = self.one.alyx.rest('tasks', 'list',
-                                            session=self.eid, graph=self.name, no_cache=True)
+        tasks_alyx_pre = self.one.alyx.rest('tasks', 'list', session=self.eid, graph=self.name, no_cache=True)
         tasks_alyx = []
         # creates all the tasks by iterating through the ordered dict
-        for k, t in self.tasks.items():
+
+        if tasks_list is not None:
+            task_items = tasks_list
+            # need to add in the session eid and the parents
+        else:
+            task_items = self.tasks.values()
+
+        for t in task_items:
             # get the parents' alyx ids to reference in the database
+            if type(t) == dict:
+                t = Bunch(t)
+                executable = t.executable
+                arguments = t.arguments
+                t['time_out_secs'] = t['time_out_sec']
+                if len(t.parents) > 0:
+                    pnames = t.parents
+            else:
+                executable = self._get_exec_name(t)
+                arguments = t.kwargs
+                if len(t.parents):
+                    pnames = [p.name for p in t.parents]
+
             if len(t.parents):
-                pnames = [p.name for p in t.parents]
                 parents_ids = [ta['id'] for ta in tasks_alyx if ta['name'] in pnames]
             else:
                 parents_ids = []
-            task_dict = {'executable': f"{t.__module__}.{t.name}", 'priority': t.priority,
+
+            task_dict = {'executable': executable, 'priority': t.priority,
                          'io_charge': t.io_charge, 'gpu': t.gpu, 'cpu': t.cpu,
                          'ram': t.ram, 'module': self.label, 'parents': parents_ids,
                          'level': t.level, 'time_out_sec': t.time_out_secs, 'session': self.eid,
-                         'status': 'Waiting', 'log': None, 'name': t.name, 'graph': self.name}
+                         'status': 'Waiting', 'log': None, 'name': t.name, 'graph': self.name,
+                         'arguments': arguments}
+            if self.data_repo:
+                task_dict.update({'data_repository': self.data_repo})
             # if the task already exists, patch it otherwise, create it
-            talyx = next(filter(lambda x: x["name"] == t.name, tasks_alyx_pre), [])
+            talyx = next(filter(lambda x: x['name'] == t.name, tasks_alyx_pre), [])
             if len(talyx) == 0:
                 talyx = self.one.alyx.rest('tasks', 'create', data=task_dict)
-            elif rerun__status__in == '__all__' or talyx['status'] in rerun__status__in:
-                talyx = self.one.alyx.rest(
-                    'tasks', 'partial_update', id=talyx['id'], data=task_dict)
+            elif talyx['status'] in rerun__status__in:
+                talyx = self.one.alyx.rest('tasks', 'partial_update', id=talyx['id'], data=task_dict)
             tasks_alyx.append(talyx)
         return tasks_alyx
 
-    def run(self, status__in=['Waiting'], machine=None, clobber=True, **kwargs):
+    def create_tasks_list_from_pipeline(self):
+        """
+        From a pipeline with tasks, creates a list of dictionaries containing task description that can be used to upload to
+        create alyx tasks
+        :return:
+        """
+        tasks_list = []
+        for k, t in self.tasks.items():
+            # get the parents' alyx ids to reference in the database
+            if len(t.parents):
+                parent_names = [p.name for p in t.parents]
+            else:
+                parent_names = []
+
+            task_dict = {'executable': self._get_exec_name(t), 'priority': t.priority,
+                         'io_charge': t.io_charge, 'gpu': t.gpu, 'cpu': t.cpu,
+                         'ram': t.ram, 'module': self.label, 'parents': parent_names,
+                         'level': t.level, 'time_out_sec': t.time_out_secs, 'session': self.eid,
+                         'status': 'Waiting', 'log': None, 'name': t.name, 'graph': self.name,
+                         'arguments': t.kwargs}
+            if self.data_repo:
+                task_dict.update({'data_repository': self.data_repo})
+
+            tasks_list.append(task_dict)
+
+        return tasks_list
+
+    def run(self, status__in=('Waiting',), machine=None, clobber=True, **kwargs):
         """
         Get all the session related jobs from alyx and run them
         :param status__in: lists of status strings to run in
@@ -465,9 +553,9 @@ class Pipeline(abc.ABC):
         :return: job_deck: list of REST dictionaries of the jobs endpoints
         :return: all_datasets: list of REST dictionaries of the dataset endpoints
         """
-        assert self.session_path, "Pipeline object has to be declared with a session path to run"
+        assert self.session_path, 'Pipeline object has to be declared with a session path to run'
         if self.one is None:
-            _logger.warning("No ONE instance found for Alyx connection, set the one property")
+            _logger.warning('No ONE instance found for Alyx connection, set the one property')
             return
         task_deck = self.one.alyx.rest('tasks', 'list', session=self.eid, no_cache=True)
         # [(t['name'], t['level']) for t in task_deck]
@@ -478,7 +566,7 @@ class Pipeline(abc.ABC):
             # here we update the status in-place to avoid another hit to the database
             task_deck[i], dsets = run_alyx_task(tdict=j, session_path=self.session_path,
                                                 one=self.one, job_deck=task_deck,
-                                                machine=machine, clobber=clobber)
+                                                machine=machine, clobber=clobber, **kwargs)
             if dsets is not None:
                 all_datasets.extend(dsets)
         return task_deck, all_datasets
@@ -487,8 +575,8 @@ class Pipeline(abc.ABC):
         return self.run(status__in=['Waiting', 'Held', 'Started', 'Errored', 'Empty'], **kwargs)
 
     def rerun(self, **kwargs):
-        return self.run(status__in=['Waiting', 'Held', 'Started', 'Errored', 'Empty', 'Complete', 'Incomplete'],
-                        **kwargs)
+
+        return self.run(status__in=[x for x in TASK_STATUS_SET if x != 'Abandoned'], **kwargs)
 
     @property
     def name(self):
@@ -496,7 +584,7 @@ class Pipeline(abc.ABC):
 
 
 def run_alyx_task(tdict=None, session_path=None, one=None, job_deck=None,
-                  max_md5_size=None, machine=None, clobber=True, location='server'):
+                  max_md5_size=None, machine=None, clobber=True, location='server', mode='log'):
     """
     Runs a single Alyx job and registers output datasets
     :param tdict:
@@ -511,6 +599,8 @@ def run_alyx_task(tdict=None, session_path=None, one=None, job_deck=None,
     :param clobber: bool, if True any existing logs are overwritten, default is True
     :param location: where you are running the task, 'server' - local lab server, 'remote' - any
     compute node/ computer, 'SDSC' - flatiron compute node, 'AWS' - using data from aws s3
+    :param mode: str ('log' or 'raise') behaviour to adopt if an error occured. If 'raise', it
+    will Raise the error at the very end of this function (ie. after having labeled the tasks)
     :return:
     """
     registered_dsets = []
@@ -522,21 +612,20 @@ def run_alyx_task(tdict=None, session_path=None, one=None, job_deck=None,
         parent_tasks = filter(lambda x: x['id'] in tdict['parents'], job_deck)
         parent_statuses = [j['status'] for j in parent_tasks]
         # if any of the parent tasks is not complete, throw a warning
-        if any(map(lambda s: s not in ['Complete', 'Incomplete'], parent_statuses)):
+        if not set(parent_statuses) <= {'Complete', 'Incomplete'}:
             _logger.warning(f"{tdict['name']} has unmet dependencies")
             # if parents are waiting or failed, set the current task status to Held
             # once the parents ran, the descendent tasks will be set from Held to Waiting (see below)
-            if any(map(lambda s: s in ['Errored', 'Held', 'Empty', 'Waiting', 'Started', 'Abandoned'],
-                       parent_statuses)):
-                tdict = one.alyx.rest('tasks', 'partial_update', id=tdict['id'],
-                                      data={'status': 'Held'})
+            if set(parent_statuses).intersection({'Errored', 'Held', 'Empty', 'Waiting', 'Started', 'Abandoned'}):
+                tdict = one.alyx.rest('tasks', 'partial_update', id=tdict['id'], data={'status': 'Held'})
             return tdict, registered_dsets
     # creates the job from the module name in the database
     exec_name = tdict['executable']
     strmodule, strclass = exec_name.rsplit('.', 1)
     classe = getattr(importlib.import_module(strmodule), strclass)
+    tkwargs = tdict.get('arguments') or {}  # if the db field is null it returns None
     task = classe(session_path, one=one, taskid=tdict['id'], machine=machine, clobber=clobber,
-                  location=location)
+                  location=location, **tkwargs)
     # sets the status flag to started before running
     one.alyx.rest('tasks', 'partial_update', id=tdict['id'], data={'status': 'Started'})
     status = task.run()
@@ -548,19 +637,25 @@ def run_alyx_task(tdict=None, session_path=None, one=None, job_deck=None,
     # otherwise register data and set (provisional) status to Complete
     else:
         try:
-            registered_dsets = task.register_datasets(one=one, max_md5_size=max_md5_size)
+            kwargs = dict(one=one, max_md5_size=max_md5_size)
+            if location == 'server':
+                # Explicitly pass lab as lab cannot be inferred from path (which the registration client tries to do).
+                # To avoid making extra REST requests we can also set labs=None if using ONE v1.20.1.
+                kwargs['labs'] = get_lab(session_path, one.alyx)
+            registered_dsets = task.register_datasets(**kwargs)
+            patch_data['status'] = 'Complete'
         except Exception:
             _logger.error(traceback.format_exc())
-            patch_data['status'] = 'Errored'
-        patch_data['status'] = 'Complete'
+            status = -1
+
     # overwrite status to errored
     if status == -1:
         patch_data['status'] = 'Errored'
     # Status -2 means a lock was encountered during run, should be rerun
-    if status == -2:
+    elif status == -2:
         patch_data['status'] = 'Waiting'
     # Status -3 should be returned if a task is Incomplete
-    if status == -3:
+    elif status == -3:
         patch_data['status'] = 'Incomplete'
     # update task status on Alyx
     t = one.alyx.rest('tasks', 'partial_update', id=tdict['id'], data=patch_data)
@@ -572,7 +667,9 @@ def run_alyx_task(tdict=None, session_path=None, one=None, job_deck=None,
         assert d['id'] != t['id'], 'task its own parent'
         # if all their parent tasks now complete, set to waiting
         parent_status = [next(x['status'] for x in job_deck if x['id'] == y) for y in d['parents']]
-        if all(x == 'Complete' for x in parent_status):
+        if set(parent_status) <= {'Complete', 'Incomplete'}:
             one.alyx.rest('tasks', 'partial_update', id=d['id'], data={'status': 'Waiting'})
     task.cleanUp()
+    if mode == 'raise' and status != 0:
+        raise ValueError(task.log)
     return t, registered_dsets
