@@ -1,6 +1,7 @@
 import logging
 from collections import OrderedDict
 
+from ibllib.pipes.base_tasks import ExperimentDescriptionRegisterRaw
 from ibllib.pipes import tasks, training_status
 from ibllib.io import ffmpeg
 from ibllib.io.extractors.base import get_session_extractor_type
@@ -10,7 +11,7 @@ from ibllib.qc.task_metrics import TaskQC, HabituationQC
 from ibllib.qc.task_extractors import TaskQCExtractor
 from ibllib.oneibl.registration import register_session_raw_data
 
-_logger = logging.getLogger('ibllib')
+_logger = logging.getLogger(__name__)
 
 
 #  level 0
@@ -32,7 +33,6 @@ class TrainingTrials(tasks.Task):
                         ('_iblrig_encoderEvents.raw*', 'raw_behavior_data', True),
                         ('_iblrig_encoderPositions.raw*', 'raw_behavior_data', True)],
         'output_files': [('*trials.goCueTrigger_times.npy', 'alf', True),
-                         ('*trials.itiDuration.npy', 'alf', False),
                          ('*trials.table.pqt', 'alf', True),
                          ('*wheel.position.npy', 'alf', True),
                          ('*wheel.timestamps.npy', 'alf', True),
@@ -67,6 +67,8 @@ class TrainingTrials(tasks.Task):
 class TrainingVideoCompress(tasks.Task):
 
     priority = 90
+    io_charge = 100
+    job_size = 'large'
 
     def _run(self):
         # avi to mp4 compression
@@ -136,6 +138,7 @@ class TrainingExtractionPipeline(tasks.Pipeline):
         tasks = OrderedDict()
         self.session_path = session_path
         # level 0
+        tasks['ExperimentDescriptionRegisterRaw'] = ExperimentDescriptionRegisterRaw(self.session_path)
         tasks['TrainingRegisterRaw'] = TrainingRegisterRaw(self.session_path)
         tasks['TrainingTrials'] = TrainingTrials(self.session_path)
         tasks['TrainingVideoCompress'] = TrainingVideoCompress(self.session_path)
