@@ -1,9 +1,8 @@
 import one.alf.io as alfio
-from one.alf.spec import is_session_path
 from one.alf.exceptions import ALFObjectNotFound
 
-from ibllib.io.raw_data_loaders import load_data
-from ibllib.oneibl.registration import _read_settings_json_compatibility_enforced, _get_session_times
+from ibllib.io.raw_data_loaders import load_bpod
+from ibllib.oneibl.registration import _get_session_times
 from ibllib.io.extractors.base import get_pipeline, get_session_extractor_type
 
 from ibllib.plots.snapshot import ReportSnapshot
@@ -31,6 +30,7 @@ TRAINING_STATUS = {'not_computed': (-2, (0, 0, 0, 0)),
 
 
 def get_trials_task(session_path, one):
+    # TODO this eventually needs to be updated for dynamic pipeline tasks
     pipeline = get_pipeline(session_path)
     if pipeline == 'training':
         from ibllib.pipes.training_preprocessing import TrainingTrials
@@ -103,10 +103,13 @@ def load_trials(sess_path, one):
         except Exception:
             try:
                 task = get_trials_task(sess_path, one=one)
-                task.run()
-                trials = alfio.load_object(sess_path.joinpath('alf'), 'trials')
-                if 'probabilityLeft' not in trials.keys():
-                    raise ALFObjectNotFound
+                if task is not None:
+                    task.run()
+                    trials = alfio.load_object(sess_path.joinpath('alf'), 'trials')
+                    if 'probabilityLeft' not in trials.keys():
+                        raise ALFObjectNotFound
+                else:
+                    trials = None
             except Exception:  # TODO how can i make this more specific
                 trials = None
     return trials
@@ -130,14 +133,24 @@ def load_combined_trials(sess_paths, one):
 
 def get_latest_training_information(sess_path, one):
     """
-    Extracts the latest training status
-    :param sess_path:
-    :param one:
-    :return:
+    Extracts the latest training status.
+
+    Parameters
+    ----------
+    sess_path : pathlib.Path
+        The session path from which to load the data.
+    one : one.api.One
+        An ONE instance.
+
+    Returns
+    -------
+    pandas.DataFrame
+        A table of training information.
     """
 
     subj_path = sess_path.parent.parent
     df = load_existing_dataframe(subj_path)
+
     # Find the dates and associated session paths where we don't have data stored in our dataframe
     missing_dates = check_up_to_date(subj_path, df)
 
@@ -156,7 +169,7 @@ def get_latest_training_information(sess_path, one):
     # Sort values by date and reset the index
     df = df.sort_values('date')
     df = df.reset_index(drop=True)
-    # Save our datagrame
+    # Save our dataframe
     save_dataframe(df, subj_path)
 
     # Now go through the backlog and compute the training status for sessions. If for example one was missing as it is cumulative
@@ -252,15 +265,27 @@ def pass_through_training_hierachy(status_new, status_old):
         return status_new
 
 
-def compute_session_duration_delay_location(sess_path):
+def compute_session_duration_delay_location(sess_path, **kwargs):
     """
     Get meta information about task. Extracts session duration, delay before session start and location of session
-    :param sess_path: session path
-    :return:
+
+    Parameters
+    ----------
+    sess_path : pathlib.Path, str
+        The session path with the pattern subject/yyyy-mm-dd/nnn.
+    task_collection : str
+        The location within the session path directory of task settings and data.
+
+    Returns
+    -------
+    int
+        The session duration in minutes, rounded to the nearest minute.
+    int
+        The delay between session start time and the first trial in seconds.
+    str {'ephys_rig', 'training_rig'}
+        The location of the session.
     """
-    settings_file = list(sess_path.glob('**/raw_behavior_data/_iblrig_taskSettings.raw*.json'))
-    md = _read_settings_json_compatibility_enforced(settings_file[0])
-    sess_data = load_data(sess_path)
+    md, sess_data = load_bpod(sess_path, **kwargs)
     start_time, end_time = _get_session_times(sess_path, md, sess_data)
     session_duration = int((end_time - start_time).total_seconds() / 60)
 
@@ -405,21 +430,25 @@ def get_training_info_for_session(session_paths, one):
 
 def check_up_to_date(subj_path, df):
     """
-    Check which sessions on local file system are missing from the computed training table
-    :param subj_path:
-    :return:
+    Check which sessions on local file system are missing from the computed training table.
+
+    Parameters
+    ----------
+    subj_path : pathlib.Path
+        The path to the subject's dated session folders.
+    df : pandas.DataFrame
+        The computed training table.
+
+    Returns
+    -------
+    pandas.DataFrame
+        A table of dates and session paths that are missing from the computed training table.
     """
-    session_dates = subj_path.glob('*')
     df_session = pd.DataFrame()
 
-    for sess_date in session_dates:
-        sess_paths = list(sess_date.glob('00*'))
-        date = sess_date.stem
-        if len(sess_paths) > 0:
-            for sess in sess_paths:
-                if is_session_path(sess):
-                    df_session = pd.concat([df_session, pd.DataFrame({'date': date, 'session_path': str(sess)}, index=[0])],
-                                           ignore_index=True)
+    for session in alfio.iter_sessions(subj_path):
+        s_df = pd.DataFrame({'date': session.parts[-2], 'session_path': str(session)}, index=[0])
+        df_session = pd.concat([df_session, s_df], ignore_index=True)
 
     if df is None or 'combined_thres_50' not in df.columns:
         return df_session
