@@ -18,9 +18,10 @@ from one.api import ONE
 import one.alf.io as alfio
 from one.alf.exceptions import ALFObjectNotFound
 from ibllib.io.video import get_video_frame, url_from_eid
-from ibllib.io import spikeglx
+import spikeglx
+import neuropixel
 from brainbox.plot import driftmap
-from brainbox.io.spikeglx import stream
+from brainbox.io.spikeglx import Streamer
 from brainbox.behavior.dlc import SAMPLING, plot_trace_on_frame, plot_wheel_position, plot_lick_hist, \
     plot_lick_raster, plot_motion_energy_hist, plot_speed_hist, plot_pupil_diameter_hist
 from brainbox.ephys_plots import image_lfp_spectrum_plot, image_rms_plot, plot_brain_regions
@@ -30,7 +31,7 @@ from iblutil.numerical import ismember
 from ibllib.plots.misc import Density
 
 
-logger = logging.getLogger('ibllib')
+logger = logging.getLogger(__name__)
 
 
 def set_axis_label_size(ax, labels=14, ticklabels=12, title=14, cmap=False):
@@ -74,17 +75,16 @@ class BehaviourPlots(ReportSnapshot):
     Behavioural plots
     """
 
-    signature = {'input_files': [('*trials.contrastLeft.npy', 'alf', True),
-                                 ('*trials.contrastRight.npy', 'alf', True),
-                                 ('*trials.feedbackType.npy', 'alf', True),
-                                 ('*trials.probabilityLeft.npy', 'alf', True),
-                                 ('*trials.choice.npy', 'alf', True),
-                                 ('*trials.response_times.npy', 'alf', True),
-                                 ('*trials.stimOn_times.npy', 'alf', True)],
-                 'output_files': [('psychometric_curve.png', 'snapshot/behaviour', True),
-                                  ('chronometric_curve.png', 'snapshot/behaviour', True),
-                                  ('reaction_time_with_trials.png', 'snapshot/behaviour', True)]
-                 }
+    signature = {
+        'input_files': [
+            ('*trials.table.pqt', 'alf', True),
+        ],
+        'output_files': [
+            ('psychometric_curve.png', 'snapshot/behaviour', True),
+            ('chronometric_curve.png', 'snapshot/behaviour', True),
+            ('reaction_time_with_trials.png', 'snapshot/behaviour', True)
+        ]
+    }
 
     def __init__(self, eid, session_path=None, one=None, **kwargs):
         self.one = one
@@ -295,11 +295,12 @@ class SpikeSorting(ReportSnapshotProbe):
     def _run(self, collection=None):
         """runs for initiated PID, streams data, destripe and check bad channels"""
 
-        def plot_driftmap(self, spikes, clusters, channels, collection, ylim=(0, 3840)):
+        def plot_driftmap(self, spikes, clusters, channels, collection):
             fig, axs = plt.subplots(1, 2, gridspec_kw={'width_ratios': [.95, .05]}, figsize=(16, 9))
             driftmap(spikes.times, spikes.depths, t_bin=0.007, d_bin=10, vmax=0.5, ax=axs[0])
             title_str = f"{self.pid_label}, {collection}, {self.pid} \n " \
                         f"{spikes.clusters.size:_} spikes, {clusters.depths.size:_} clusters"
+            ylim = (0, np.max(channels['axial_um']))
             axs[0].set(ylim=ylim, title=title_str)
             run_label = str(Path(collection).relative_to(f'alf/{self.pname}'))
             run_label = "ks2matlab" if run_label == '.' else run_label
@@ -325,6 +326,7 @@ class SpikeSorting(ReportSnapshotProbe):
             spikes = alfio.load_object(self.session_path.joinpath(collection), 'spikes')
             clusters = alfio.load_object(self.session_path.joinpath(collection), 'clusters')
             channels = alfio.load_object(self.session_path.joinpath(collection), 'channels')
+            channels['axial_um'] = channels['localCoordinates'][:, 1]
 
             out, fig, axs = plot_driftmap(self, spikes, clusters, channels, collection)
             output_files.append(out)
@@ -422,26 +424,40 @@ class BadChannelsAp(ReportSnapshotProbe):
             else:
                 electrodes = None
 
-            sr, t0 = stream(self.pid, T0, nsecs=1, one=self.one)
-            raw = sr[:, :-sr.nsync].T
+            nsecs = 1
+            sr = Streamer(pid=self.pid, one=self.one, remove_cached=False, typ='ap')
+            s0 = T0 * sr.fs
+            tsel = slice(int(s0), int(s0) + int(nsecs * sr.fs))
+            # Important: remove sync channel from raw data, and transpose
+            raw = sr[tsel, :-sr.nsync].T
+
         else:
             electrodes = None
             ap_file = next(self.session_path.joinpath('raw_ephys_data', self.pname).glob('*ap.*bin'), None)
             if ap_file is not None:
                 sr = spikeglx.Reader(ap_file)
+                # If T0 is greater than recording length, take 500 sec before end
+                if sr.rl < T0:
+                    T0 = int(sr.rl - 500)
                 raw = sr[int((sr.fs * T0)):int((sr.fs * (T0 + 1))), :-sr.nsync].T
             else:
                 return []
 
+        if sr.meta.get('NP2.4_shank', None) is not None:
+            h = neuropixel.trace_header(sr.major_version, nshank=4)
+            h = neuropixel.split_trace_header(h, shank=int(sr.meta.get('NP2.4_shank')))
+        else:
+            h = neuropixel.trace_header(sr.major_version, nshank=np.unique(sr.geometry['shank']).size)
+
         channel_labels, channel_features = voltage.detect_bad_channels(raw, sr.fs)
         _, eqcs, output_files = ephys_bad_channels(
-            raw=raw, fs=sr.fs, channel_labels=channel_labels, channel_features=channel_features, channels=electrodes,
+            raw=raw, fs=sr.fs, channel_labels=channel_labels, channel_features=channel_features, h=h, channels=electrodes,
             title=SNAPSHOT_LABEL, destripe=True, save_dir=self.output_directory, br=self.brain_regions, pid_info=self.pid_label)
         self.eqcs = eqcs
         return output_files
 
 
-def ephys_bad_channels(raw, fs, channel_labels, channel_features, channels=None, title="ephys_bad_channels",
+def ephys_bad_channels(raw, fs, channel_labels, channel_features, h=None, channels=None, title="ephys_bad_channels",
                        save_dir=None, destripe=False, eqcs=None, br=None, pid_info=None, plot_backend='matplotlib'):
     nc, ns = raw.shape
     rl = ns / fs
@@ -480,7 +496,7 @@ def ephys_bad_channels(raw, fs, channel_labels, channel_features, channels=None,
         eqcs.append(Density(butt, fs=fs, taxis=1, ax=axs[0], title='highpass', vmin=eqc_levels[0], vmax=eqc_levels[1]))
 
         if destripe:
-            dest = voltage.destripe(raw, fs=fs, channel_labels=channel_labels)
+            dest = voltage.destripe(raw, fs=fs, h=h, channel_labels=channel_labels)
             _, axs = plt.subplots(1, 2, gridspec_kw={'width_ratios': [.95, .05]}, figsize=(16, 9))
             eqcs.append(Density(
                 dest, fs=fs, taxis=1, ax=axs[0], title='destripe', vmin=eqc_levels[0], vmax=eqc_levels[1]))
@@ -515,7 +531,7 @@ def ephys_bad_channels(raw, fs, channel_labels, channel_features, channels=None,
         eqcs.append(viewephys(butt, fs=fs, channels=channels, title='highpass', br=br))
 
         if destripe:
-            dest = voltage.destripe(raw, fs=fs, channel_labels=channel_labels)
+            dest = voltage.destripe(raw, fs=fs, h=h, channel_labels=channel_labels)
             eqcs.append(viewephys(dest, fs=fs, channels=channels, title='destripe', br=br))
             eqcs.append(viewephys((butt - dest), fs=fs, channels=channels, title='difference', br=br))
 
