@@ -35,11 +35,15 @@ Run the QC for all cameras
 import logging
 from inspect import getmembers, isfunction
 from pathlib import Path
+from itertools import chain
+import copy
 
 import cv2
 import numpy as np
 import matplotlib.pyplot as plt
+import pandas as pd
 from matplotlib.patches import Rectangle
+from labcams import parse_cam_log
 
 import one.alf.io as alfio
 from one.util import filter_datasets
@@ -53,16 +57,19 @@ from ibllib.io.extractors import ephys_fpga, training_wheel
 from ibllib.io.extractors.video_motion import MotionAlignment
 from ibllib.io.extractors.base import get_session_extractor_type
 from ibllib.io import raw_data_loaders as raw
+from ibllib.io.session_params import read_params, get_sync
 import brainbox.behavior.wheel as wh
 from ibllib.io.video import get_video_meta, get_video_frames_preload, assert_valid_label
 from . import base
 
-_log = logging.getLogger('ibllib')
+_log = logging.getLogger(__name__)
 
 
 class CameraQC(base.QC):
     """A class for computing camera QC metrics"""
     dstypes = [
+        '_ibl_experiment.description',
+        '_iblrig_Camera.frameData',  # Replaces the next 3 datasets
         '_iblrig_Camera.frame_counter',
         '_iblrig_Camera.GPIO',
         '_iblrig_Camera.timestamps',
@@ -124,6 +131,8 @@ class CameraQC(base.QC):
         self.download_data = kwargs.pop('download_data', download_data)
         self.stream = kwargs.pop('stream', None)
         self.n_samples = kwargs.pop('n_samples', 100)
+        self.sync_collection = kwargs.pop('sync_collection', None)
+        self.sync = kwargs.pop('sync_type', None)
         super().__init__(session_path_or_eid, **kwargs)
 
         # Data
@@ -141,8 +150,6 @@ class CameraQC(base.QC):
                 _log.error('No remote or local video file found')
                 self.video_path = None
 
-        logging.disable(logging.CRITICAL)
-        self._type = get_session_extractor_type(self.session_path) or None
         logging.disable(logging.NOTSET)
         keys = ('count', 'pin_state', 'audio', 'fpga_times', 'wheel', 'video',
                 'frame_samples', 'timestamps', 'camera_times', 'bonsai_times')
@@ -153,6 +160,10 @@ class CameraQC(base.QC):
         self.metrics = None
         self.outcome = 'NOT_SET'
 
+        # Specify any checks to remove
+        self.checks_to_remove = []
+        self._type = None
+
     @property
     def type(self):
         """
@@ -164,8 +175,7 @@ class CameraQC(base.QC):
         else:
             return 'ephys' if 'ephys' in self._type else 'training'
 
-    def load_data(self, download_data: bool = None,
-                  extract_times: bool = False, load_video: bool = True) -> None:
+    def load_data(self, download_data: bool = None, extract_times: bool = False, load_video: bool = True) -> None:
         """Extract the data from raw data files
         Extracts all the required task data from the raw data files.
 
@@ -197,17 +207,30 @@ class CameraQC(base.QC):
         self.data['count'], self.data['pin_state'] = \
             raw.load_embedded_frame_data(self.session_path, self.label, raw=True)
 
+        # If there is an experiment description and there are video parameters
+        sess_params = read_params(self.session_path) or {}
+        task_collection = get_task_collection(sess_params)
+        self._set_sync(sess_params)
+        if not self.sync:
+            if not self.type:
+                self._type = get_session_extractor_type(self.session_path, task_collection)
+            self.sync = 'nidq' if 'ephys' in self.type else 'bpod'
+        self._update_meta_from_session_params(sess_params)
+
         # Load the audio and raw FPGA times
-        if self.type == 'ephys':
-            sync, chmap = ephys_fpga.get_main_probe_sync(self.session_path)
+        if self.sync != 'bpod' and self.sync is not None:
+            self.sync_collection = self.sync_collection or 'raw_ephys_data'
+            sync, chmap = ephys_fpga.get_sync_and_chn_map(self.session_path, self.sync_collection)
             audio_ttls = ephys_fpga.get_sync_fronts(sync, chmap['audio'])
             self.data['audio'] = audio_ttls['times']  # Get rises
             # Load raw FPGA times
             cam_ts = extract_camera_sync(sync, chmap)
             self.data['fpga_times'] = cam_ts[self.label]
         else:
-            bpod_data = raw.load_data(self.session_path)
-            _, audio_ttls = raw.load_bpod_fronts(self.session_path, bpod_data)
+            self.sync_collection = self.sync_collection or task_collection
+            bpod_data = raw.load_data(self.session_path, task_collection)
+            _, audio_ttls = raw.load_bpod_fronts(
+                self.session_path, data=bpod_data, task_collection=task_collection)
             self.data['audio'] = audio_ttls['times']
 
         # Load extracted frame times
@@ -218,9 +241,10 @@ class CameraQC(base.QC):
                 alf_path, f'{self.label}Camera', short_keys=True)['times']
         except AssertionError:  # Re-extract
             kwargs = dict(video_path=self.video_path, labels=self.label)
-            if self.type == 'ephys':
+            if self.sync != 'bpod' and self.sync is not None:
                 kwargs = {**kwargs, 'sync': sync, 'chmap': chmap}  # noqa
-            outputs, _ = extract_all(self.session_path, self.type, save=False, **kwargs)
+            outputs, _ = extract_all(self.session_path, self.sync, save=False,
+                                     sync_collection=self.sync_collection, **kwargs)
             self.data['timestamps'] = outputs[f'{self.label}_camera_timestamps']
         except ALFObjectNotFound:
             _log.warning('no camera.times ALF found for session')
@@ -228,13 +252,16 @@ class CameraQC(base.QC):
         # Get audio and wheel data
         wheel_keys = ('timestamps', 'position')
         try:
+            # glob in case wheel data are in sub-collections
+            alf_path = next(alf_path.rglob('*wheel.timestamps*')).parent
             self.data['wheel'] = alfio.load_object(alf_path, 'wheel', short_keys=True)
-        except ALFObjectNotFound:
+        except (StopIteration, ALFObjectNotFound):
             # Extract from raw data
-            if self.type == 'ephys':
+            if self.sync != 'bpod' and self.sync is not None:
                 wheel_data = ephys_fpga.extract_wheel_sync(sync, chmap)
             else:
-                wheel_data = training_wheel.get_wheel_position(self.session_path)
+                wheel_data = training_wheel.get_wheel_position(
+                    self.session_path, task_collection=task_collection)
             self.data['wheel'] = Bunch(zip(wheel_keys, wheel_data))
 
         # Find short period of wheel motion for motion correlation.
@@ -304,11 +331,27 @@ class CameraQC(base.QC):
         :return:
         """
         assert self.one is not None, 'ONE required to download data'
-        # dataset collections outside this list are ignored (e.g. probe00, raw_passive_data)
-        collections = ('alf', 'raw_ephys_data', 'raw_behavior_data', 'raw_video_data')
+
+        sess_params = {}
+        if self.download_data:
+            dset = self.one.list_datasets(self.session_path, '*experiment.description*', details=True)
+            if self.one._check_filesystem(dset):
+                sess_params = read_params(self.session_path) or {}
+        else:
+            sess_params = read_params(self.session_path) or {}
+        self._set_sync(sess_params)
+
         # Get extractor type
         is_ephys = 'ephys' in (self.type or self.one.get_details(self.eid)['task_protocol'])
-        dtypes = self.dstypes + self.dstypes_fpga if is_ephys else self.dstypes
+        self.sync = self.sync or ('nidq' if is_ephys else 'bpod')
+
+        is_fpga = 'bpod' not in self.sync
+
+        # dataset collections outside this list are ignored (e.g. probe00, raw_passive_data)
+        collections = (
+            'alf', '', get_task_collection(sess_params), get_video_collection(sess_params, self.label)
+        )
+        dtypes = self.dstypes + self.dstypes_fpga if is_fpga else self.dstypes
         assert_unique = True
         # Check we have raw ephys data for session
         if is_ephys and len(self.one.list_datasets(self.eid, collection='raw_ephys_data')) == 0:
@@ -316,23 +359,35 @@ class CameraQC(base.QC):
             det = self.one.get_details(self.eid, full=True)
             probe_model = next(x['model'] for x in det['probe_insertion'])
             assert probe_model == '3A', 'raw ephys data missing'
-            collections += ('raw_ephys_data/probe00', 'raw_ephys_data/probe01')
+            collections += (self.sync_collection or 'raw_ephys_data',)
+            if sess_params:
+                probes = sess_params.get('devices', {}).get('neuropixel', {})
+                probes = set(x.get('collection') for x in chain(*map(dict.values, probes)))
+                collections += tuple(probes)
+            else:
+                collections += ('raw_ephys_data/probe00', 'raw_ephys_data/probe01')
             assert_unique = False
         for dstype in dtypes:
             datasets = self.one.type2datasets(self.eid, dstype, details=True)
             if 'camera' in dstype.lower():  # Download individual camera file
                 datasets = filter_datasets(datasets, filename=f'.*{self.label}.*')
             else:  # Ignore probe datasets, etc.
-                datasets = filter_datasets(datasets, collection=collections,
-                                           assert_unique=assert_unique)
+                _datasets = filter_datasets(datasets, collection=collections, assert_unique=assert_unique)
+                if '' in collections:  # Must be handled as a separate query
+                    datasets = filter_datasets(datasets, collection='', assert_unique=assert_unique)
+                    datasets = pd.concat([datasets, _datasets]).sort_index()
+                else:
+                    datasets = _datasets
+
             if any(x.endswith('.mp4') for x in datasets.rel_path) and self.stream:
                 names = [x.split('/')[-1] for x in self.one.list_datasets(self.eid, details=False)]
                 assert f'_iblrig_{self.label}Camera.raw.mp4' in names, 'No remote video file found'
                 continue
-            optional = ('camera.times', '_iblrig_Camera.raw', 'wheel.position',
-                        'wheel.timestamps', '_iblrig_Camera.frame_counter', '_iblrig_Camera.GPIO')
+            optional = ('camera.times', '_iblrig_Camera.raw', 'wheel.position', 'wheel.timestamps',
+                        '_iblrig_Camera.timestamps', '_iblrig_Camera.frame_counter', '_iblrig_Camera.GPIO',
+                        '_iblrig_Camera.frameData', '_ibl_experiment.description')
             present = (
-                self.one._download_datasets(datasets)
+                self.one._check_filesystem(datasets)
                 if self.download_data
                 else (next(self.session_path.rglob(d), None) for d in datasets['rel_path'])
             )
@@ -340,7 +395,56 @@ class CameraQC(base.QC):
             required = (dstype not in optional)
             all_present = not datasets.empty and all(present)
             assert all_present or not required, f'Dataset {dstype} not found'
-        self._type = get_session_extractor_type(self.session_path)
+
+        if not self.type and self.sync != 'nidq':
+            self._type = get_session_extractor_type(self.session_path)
+
+    def _set_sync(self, session_params=False):
+        """Set the sync and sync_collection attributes if not already set.
+
+        Also set the type attribute based on the sync. NB The type attribute is for legacy sessions.
+
+        Parameters
+        ----------
+        session_params : dict, bool
+            The loaded experiment description file.  If False, attempts to load it from the session_path.
+        """
+        if session_params is False:
+            if not self.session_path:
+                raise ValueError('No session path set')
+            session_params = read_params(self.session_path)
+        sync, sync_dict = get_sync(session_params)
+        self.sync = self.sync or sync
+        self.sync_collection = self.sync_collection or sync_dict.get('collection')
+        if self.sync:
+            self._type = 'ephys' if self.sync == 'nidq' else 'training'
+
+    def _update_meta_from_session_params(self, sess_params):
+        """
+        Update the default expected video properties with those defined in the experiment
+        description file (if any).  This updates the `video_meta` property with the fps, width and
+        height for the type and camera label.
+
+        Parameters
+        ----------
+        sess_params : dict
+            The loaded experiment.description file.
+        """
+        try:
+            assert sess_params
+            video_pars = sess_params.get('devices', {}).get('cameras', {}).get(self.label)
+        except (AssertionError, KeyError):
+            return
+        PROPERTIES = ('width', 'height', 'fps')
+        video_meta = copy.deepcopy(self.video_meta)  # must re-assign as it's a class attribute
+        if self.type not in video_meta:
+            video_meta[self.type] = {}
+        if self.label not in video_meta[self.type]:
+            video_meta[self.type][self.label] = {}
+        video_meta[self.type][self.label].update(
+            **{k: v for k, v in video_pars.items() if k in PROPERTIES}
+        )
+        self.video_meta = video_meta
 
     def run(self, update: bool = False, **kwargs) -> (str, dict):
         """
@@ -362,8 +466,12 @@ class CameraQC(base.QC):
 
         def is_metric(x):
             return isfunction(x) and x.__name__.startswith('check_')
+        # import importlib
+        # classe = getattr(importlib.import_module(self.__module__), self.__name__)
+        # print(classe)
 
-        checks = getmembers(CameraQC, is_metric)
+        checks = getmembers(self.__class__, is_metric)
+        checks = self.remove_check(checks)
         self.metrics = {f'_{namespace}_' + k[6:]: fn(self) for k, fn in checks}
 
         values = [x if isinstance(x, str) else x[0] for x in self.metrics.values()]
@@ -372,15 +480,23 @@ class CameraQC(base.QC):
 
         if update:
             extended = {
-                k: None if v is None or v == 'NOT_SET'
-                else base.CRITERIA[v] < 3 if isinstance(v, str)
-                else (base.CRITERIA[v[0]] < 3, *v[1:])  # Convert first value to bool if array
+                k: 'NOT_SET' if v is None else v
                 for k, v in self.metrics.items()
             }
             self.update_extended_qc(extended)
             self.update(outcome, namespace)
         return outcome, self.metrics
 
+    def remove_check(self, checks):
+        if len(self.checks_to_remove) == 0:
+            return checks
+        else:
+            for check in self.checks_to_remove:
+                check_names = [ch[0] for ch in checks]
+                idx = check_names.index(check)
+                checks.pop(idx)
+            return checks
+
     def check_brightness(self, bounds=(40, 200), max_std=20, roi=True, display=False):
         """Check that the video brightness is within a given range
         The mean brightness of each frame must be with the bounds provided, and the standard
@@ -466,8 +582,7 @@ class CameraQC(base.QC):
         return 'PASS' if abs(Fs - fps) < threshold else 'FAIL', float(round(Fs, 3))
 
     def check_pin_state(self, display=False):
-        """Check the pin state reflects Bpod TTLs
-        """
+        """Check the pin state reflects Bpod TTLs"""
         if not data_for_keys(('video', 'pin_state', 'audio'), self.data):
             return 'NOT_SET'
         size_diff = int(self.data['pin_state'].shape[0] - self.data['video']['length'])
@@ -475,7 +590,7 @@ class CameraQC(base.QC):
         low2high = np.insert(np.diff(self.data['pin_state'][:, -1].astype(int)) == 1, 0, False)
         # NB: Time between two consecutive TTLs can be sub-frame, so this will fail
         ndiff_low2high = int(self.data['audio'][::2].size - sum(low2high))
-        state_ttl_matches = ndiff_low2high == 0
+        # state_ttl_matches = ndiff_low2high == 0
         # Check within ms of audio times
         if display:
             plt.Figure()
@@ -490,7 +605,7 @@ class CameraQC(base.QC):
 
         outcome = self.overall_outcome(
             ('PASS' if size_diff == 0 else 'WARNING' if np.abs(size_diff) < 5 else 'FAIL',
-             'PASS' if state_ttl_matches else 'WARNING')
+             'PASS' if np.abs(ndiff_low2high) < 5 else 'WARNING')
         )
         return outcome, ndiff_low2high, size_diff
 
@@ -521,18 +636,13 @@ class CameraQC(base.QC):
         """Check that the camera.times array is reasonable"""
         if not data_for_keys(('timestamps', 'video'), self.data):
             return 'NOT_SET'
-        # Check frame rate matches what we expect
-        expected = 1 / self.video_meta[self.type][self.label]['fps']
-        # TODO Remove dropped frames from test
-        frame_delta = np.diff(self.data['timestamps'])
-        fps_matches = np.isclose(np.median(frame_delta), expected, atol=0.01)
         # Check number of timestamps matches video
         length_matches = self.data['timestamps'].size == self.data['video'].length
         # Check times are strictly increasing
         increasing = all(np.diff(self.data['timestamps']) > 0)
         # Check times do not contain nans
         nanless = not np.isnan(self.data['timestamps']).any()
-        return 'PASS' if increasing and fps_matches and length_matches and nanless else 'FAIL'
+        return 'PASS' if increasing and length_matches and nanless else 'FAIL'
 
     def check_camera_times(self):
         """Check that the number of raw camera timestamps matches the number of video frames"""
@@ -557,10 +667,28 @@ class CameraQC(base.QC):
 
         Check is skipped for body camera videos as the wheel is often obstructed
 
-        :param tolerance: maximum absolute offset in frames.  If two values, the maximum value
-        is taken as the warning threshold
-        :param display: if true, the wheel motion energy is plotted against the rotary encoder
-        :returns: outcome string, frame offset
+        Parameters
+        ----------
+        tolerance : int, (int, int)
+            Maximum absolute offset in frames.  If two values, the maximum value is taken as the
+            warning threshold.
+        display : bool
+            If true, the wheel motion energy is plotted against the rotary encoder.
+
+        Returns
+        -------
+        str
+            The outcome string, one of {'NOT_SET', 'FAIL', 'WARNING', 'PASS'}.
+        int
+            Frame offset, i.e. by how many frames the video was shifted to match the rotary encoder
+            signal.  Negative values mean the video was shifted backwards with respect to the wheel
+            timestamps.
+
+        Notes
+        -----
+        - A negative frame offset typically means that there were frame TTLs at the beginning that
+        do not correspond to any video frames (sometimes the first few frames aren't saved to
+        disk).  Since 2021-09-15 the extractor should compensate for this.
         """
         wheel_present = data_for_keys(('position', 'timestamps', 'period'), self.data['wheel'])
         if not wheel_present or self.label == 'body':
@@ -907,11 +1035,237 @@ class CameraQC(base.QC):
         return ax
 
 
+class CameraQCCamlog(CameraQC):
+    """A class for computing camera QC metrics from camlog data. For this QC we expect the check_pin_state to be NOT_SET as we are
+    not using the GPIO for timestamp alignment"""
+    dstypes = [
+        '_iblrig_taskData.raw',
+        '_iblrig_taskSettings.raw',
+        '_iblrig_Camera.raw',
+        'camera.times',
+        'wheel.position',
+        'wheel.timestamps'
+    ]
+    dstypes_fpga = [
+        '_spikeglx_sync.channels',
+        '_spikeglx_sync.polarities',
+        '_spikeglx_sync.times',
+        'DAQData.raw.meta',
+        'DAQData.wiring'
+    ]
+
+    def __init__(self, session_path_or_eid, camera, sync_collection='raw_sync_data', sync_type='nidq', **kwargs):
+        super().__init__(session_path_or_eid, camera, sync_collection=sync_collection, sync_type=sync_type, **kwargs)
+        self._type = 'ephys'
+        self.checks_to_remove = ['check_pin_state']
+
+    def load_data(self, download_data: bool = None,
+                  extract_times: bool = False, load_video: bool = True, **kwargs) -> None:
+        """Extract the data from raw data files
+        Extracts all the required task data from the raw data files.
+
+        Data keys:
+            - count (int array): the sequential frame number (n, n+1, n+2...)
+            - pin_state (): the camera GPIO pin; records the audio TTLs; should be one per frame
+            - audio (float array): timestamps of audio TTL fronts
+            - fpga_times (float array): timestamps of camera TTLs recorded by FPGA
+            - timestamps (float array): extracted video timestamps (the camera.times ALF)
+            - bonsai_times (datetime array): system timestamps of video PC; should be one per frame
+            - camera_times (float array): camera frame timestamps extracted from frame headers
+            - wheel (Bunch): rotary encoder timestamps, position and period used for wheel motion
+            - video (Bunch): video meta data, including dimensions and FPS
+            - frame_samples (h x w x n array): array of evenly sampled frames (1 colour channel)
+
+        :param download_data: if True, any missing raw data is downloaded via ONE.
+        Missing data will raise an AssertionError
+        :param extract_times: if True, the camera.times are re-extracted from the raw data
+        :param load_video: if True, calls the load_video_data method
+        """
+        assert self.session_path, 'no session path set'
+        if download_data is not None:
+            self.download_data = download_data
+        if self.download_data and self.eid and self.one and not self.one.offline:
+            self.ensure_required_data()
+        _log.info('Gathering data for QC')
+
+        # If there is an experiment description and there are video parameters
+        sess_params = read_params(self.session_path) or {}
+        video_collection = get_video_collection(sess_params, self.label)
+        task_collection = get_task_collection(sess_params)
+        self._set_sync(sess_params)
+        self._update_meta_from_session_params(sess_params)
+
+        # Get frame count
+        log, _ = parse_cam_log(self.session_path.joinpath(video_collection, f'_iblrig_{self.label}Camera.raw.camlog'))
+        self.data['count'] = log.frame_id.values
+
+        # Load the audio and raw FPGA times
+        if self.sync != 'bpod' and self.sync is not None:
+            sync, chmap = ephys_fpga.get_sync_and_chn_map(self.session_path, self.sync_collection)
+            audio_ttls = ephys_fpga.get_sync_fronts(sync, chmap['audio'])
+            self.data['audio'] = audio_ttls['times']  # Get rises
+            # Load raw FPGA times
+            cam_ts = extract_camera_sync(sync, chmap)
+            self.data['fpga_times'] = cam_ts[self.label]
+        else:
+            bpod_data = raw.load_data(self.session_path, task_collection=task_collection)
+            _, audio_ttls = raw.load_bpod_fronts(self.session_path, data=bpod_data, task_collection=task_collection)
+            self.data['audio'] = audio_ttls['times']
+
+        # Load extracted frame times
+        alf_path = self.session_path / 'alf'
+        try:
+            assert not extract_times
+            self.data['timestamps'] = alfio.load_object(
+                alf_path, f'{self.label}Camera', short_keys=True)['times']
+        except AssertionError:  # Re-extract
+            kwargs = dict(video_path=self.video_path, labels=self.label)
+            if self.sync == 'bpod':
+                kwargs = {**kwargs, 'task_collection': task_collection}
+            else:
+                kwargs = {**kwargs, 'sync': sync, 'chmap': chmap}  # noqa
+            outputs, _ = extract_all(self.session_path, self.sync, save=False, camlog=True, **kwargs)
+            self.data['timestamps'] = outputs[f'{self.label}_camera_timestamps']
+        except ALFObjectNotFound:
+            _log.warning('no camera.times ALF found for session')
+
+        # Get audio and wheel data
+        wheel_keys = ('timestamps', 'position')
+        try:
+            # glob in case wheel data are in sub-collections
+            alf_path = next(alf_path.rglob('*wheel.timestamps*')).parent
+            self.data['wheel'] = alfio.load_object(alf_path, 'wheel', short_keys=True)
+        except ALFObjectNotFound:
+            # Extract from raw data
+            if self.sync != 'bpod':
+                wheel_data = ephys_fpga.extract_wheel_sync(sync, chmap)
+            else:
+                wheel_data = training_wheel.get_wheel_position(self.session_path, task_collection=task_collection)
+            self.data['wheel'] = Bunch(zip(wheel_keys, wheel_data))
+
+        # Find short period of wheel motion for motion correlation.
+        if data_for_keys(wheel_keys, self.data['wheel']) and self.data['timestamps'] is not None:
+            self.data['wheel'].period = self.get_active_wheel_period(self.data['wheel'])
+
+        # load in camera times
+        self.data['camera_times'] = log.timestamp.values
+
+        # Gather information from video file
+        if load_video:
+            _log.info('Inspecting video file...')
+            self.load_video_data()
+
+    def ensure_required_data(self):
+        """
+        Ensures the datasets required for QC are local.  If the download_data attribute is True,
+        any missing data are downloaded.  If all the data are not present locally at the end of
+        it an exception is raised.  If the stream attribute is True, the video file is not
+        required to be local, however it must be remotely accessible.
+        NB: Requires a valid instance of ONE and a valid session eid.
+        :return:
+        """
+        assert self.one is not None, 'ONE required to download data'
+
+        sess_params = {}
+        if self.download_data:
+            dset = self.one.list_datasets(self.session_path, '*experiment.description*', details=True)
+            if self.one._check_filesystem(dset):
+                sess_params = read_params(self.session_path) or {}
+        else:
+            sess_params = read_params(self.session_path) or {}
+        self._set_sync(sess_params)
+
+        # dataset collections outside this list are ignored (e.g. probe00, raw_passive_data)
+        collections = (
+            'alf', self.sync_collection, get_task_collection(sess_params),
+            get_video_collection(sess_params, self.label))
+
+        # Get extractor type
+        dtypes = self.dstypes + self.dstypes_fpga
+        assert_unique = True
+
+        for dstype in dtypes:
+            datasets = self.one.type2datasets(self.eid, dstype, details=True)
+            if 'camera' in dstype.lower():  # Download individual camera file
+                datasets = filter_datasets(datasets, filename=f'.*{self.label}.*')
+            else:  # Ignore probe datasets, etc.
+                datasets = filter_datasets(datasets, collection=collections,
+                                           assert_unique=assert_unique)
+            if any(x.endswith('.mp4') for x in datasets.rel_path) and self.stream:
+                names = [x.split('/')[-1] for x in self.one.list_datasets(self.eid, details=False)]
+                assert f'_iblrig_{self.label}Camera.raw.mp4' in names, 'No remote video file found'
+                continue
+            optional = ('camera.times', '_iblrig_Camera.raw', 'wheel.position', 'wheel.timestamps')
+            present = (
+                self.one._check_filesystem(datasets)
+                if self.download_data
+                else (next(self.session_path.rglob(d), None) for d in datasets['rel_path'])
+            )
+
+            required = (dstype not in optional)
+            all_present = not datasets.empty and all(present)
+            assert all_present or not required, f'Dataset {dstype} not found'
+
+    def check_camera_times(self):
+        """Check that the number of raw camera timestamps matches the number of video frames"""
+        if not data_for_keys(('camera_times', 'video'), self.data):
+            return 'NOT_SET'
+        length_match = len(self.data['camera_times']) == self.data['video'].length
+        outcome = 'PASS' if length_match else 'WARNING'
+        # 1 / np.median(np.diff(self.data.camera_times))
+        return outcome, len(self.data['camera_times']) - self.data['video'].length
+
+
 def data_for_keys(keys, data):
     """Check keys exist in 'data' dict and contain values other than None"""
     return data is not None and all(k in data and data.get(k, None) is not None for k in keys)
 
 
+def get_task_collection(sess_params):
+    """
+    Returns the first task collection from the experiment description whose task name does not
+    contain 'passive', otherwise returns 'raw_behavior_data'.
+
+    Parameters
+    ----------
+    sess_params : dict
+        The loaded experiment description file.
+
+    Returns
+    -------
+    str:
+        The collection presumed to contain wheel data.
+    """
+    sess_params = sess_params or {}
+    tasks = (chain(*map(dict.items, sess_params.get('tasks', []))))
+    return next((v['collection'] for k, v in tasks if 'passive' not in k), 'raw_behavior_data')
+
+
+def get_video_collection(sess_params, label):
+    """
+    Returns the collection containing the raw video data for a given camera.
+
+    Parameters
+    ----------
+    sess_params : dict
+        The loaded experiment description file.
+    label : str
+        The camera label.
+
+    Returns
+    -------
+    str:
+        The collection presumed to contain the video data.
+    """
+    DEFAULT = 'raw_video_data'
+    value = sess_params or {}
+    for key in ('devices', 'cameras', label, 'collection'):
+        value = value.get(key)
+        if not value:
+            return DEFAULT
+    return value
+
+
 def run_all_qc(session, cameras=('left', 'right', 'body'), **kwargs):
     """Run QC for all cameras
     Run the camera QC for left, right and body cameras.
@@ -923,9 +1277,12 @@ def run_all_qc(session, cameras=('left', 'right', 'body'), **kwargs):
     :return: dict of CameraCQ objects
     """
     qc = {}
+    camlog = kwargs.pop('camlog', False)
+    CamQC = CameraQCCamlog if camlog else CameraQC
+
     run_args = {k: kwargs.pop(k) for k in ('download_data', 'extract_times', 'update')
                 if k in kwargs.keys()}
     for camera in cameras:
-        qc[camera] = CameraQC(session, camera, **kwargs)
+        qc[camera] = CamQC(session, camera, **kwargs)
         qc[camera].run(**run_args)
     return qc
