@@ -1,5 +1,4 @@
 import functools
-import logging
 import shutil
 import tempfile
 import unittest
@@ -55,7 +54,6 @@ class TestExtractTrialData(unittest.TestCase):
         # turn off logging for unit testing as we will purposedly go into warning/error cases
         self.wheel_ge5_path = self.main_path / 'data' / 'wheel_ge5'
         self.wheel_lt5_path = self.main_path / 'data' / 'wheel_lt5'
-        self.logger = logging.getLogger('ibllib')
         # Save some dummy wheel moves data for trial firstMovement_times extraction
 
     def test_get_feedbackType(self):
@@ -335,19 +333,6 @@ class TestExtractTrialData(unittest.TestCase):
         self.assertTrue(isinstance(di, np.ndarray))
         self.assertFalse(np.isnan(di).all())
 
-    def test_get_iti_duration(self):
-        # TRAINING SESSIONS
-        iti = training_trials.ItiDuration(
-            self.training_lt5['path']).extract()[0]
-        self.assertTrue(isinstance(iti, np.ndarray))
-        # -- version >= 5.0.0 iti always == 0.5 sec no extract
-
-        # BIASED SESSIONS
-        iti = biased_trials.ItiDuration(
-            self.biased_lt5['path']).extract()[0]
-        self.assertTrue(isinstance(iti, np.ndarray))
-        # -- version >= 5.0.0 iti always == 0.5 sec no extract
-
     def test_get_response_times(self):
         # TRAINING SESSIONS
         rt = training_trials.ResponseTimes(
@@ -457,11 +442,16 @@ class TestExtractTrialData(unittest.TestCase):
     @wheelMoves_fixture
     def test_extract_all(self):
         # TRAINING SESSIONS
-        out, files = training_trials.extract_all(
-            self.training_lt5['path'], settings={'IBLRIG_VERSION_TAG': '4.9.9'}, save=True)
+        # Expect an error raised because no wheel moves were present in test data
+        with self.assertRaises(ValueError) as ex:
+            training_trials.extract_all(
+                self.training_lt5['path'], settings={'IBLRIG_VERSION_TAG': '4.9.9'}, save=True)
+            self.assertIn('_ibl_wheelMoves.intervals.npy appears to be empty', str(ex.exception))
         # -- version >= 5.0.0
-        out, files = training_trials.extract_all(
-            self.training_ge5['path'], save=True)
+        out, files = training_trials.extract_all(self.training_ge5['path'], save=True)
+        self.assertEqual(19, len(out))
+        self.assertTrue(all(map(Path.exists, files)))
+
         # BIASED SESSIONS
         # The new trials extractor additionally extracts the wheel data and this fails for the < 5.0
         # test data so we will stub the wheel extractor
@@ -470,9 +460,12 @@ class TestExtractTrialData(unittest.TestCase):
             Wheel().extract.return_value = ({}, [])
             out, files = biased_trials.extract_all(
                 self.biased_lt5['path'], settings={'IBLRIG_VERSION_TAG': '4.9.9'}, save=True)
+            self.assertEqual(15, len(out))
+            self.assertTrue(all(map(Path.exists, files)))
         # -- version >= 5.0.0
-        out, files = biased_trials.extract_all(
-            self.biased_ge5['path'], save=True)
+        out, files = biased_trials.extract_all(self.biased_ge5['path'], save=True)
+        self.assertEqual(19, len(out))
+        self.assertTrue(all(map(Path.exists, files)))
 
     def test_encoder_positions_clock_reset(self):
         # TRAINING SESSIONS
@@ -531,20 +524,31 @@ class TestExtractTrialData(unittest.TestCase):
 
     def test_size_outputs(self):
         # check the output dimensions
+        # VERSION >= 5.0.0
         from ibllib.io.extractors.bpod_trials import extract_all
         extract_all(self.training_ge5['path'])
         trials = alfio.load_object(self.training_ge5['path'] / 'alf', object='trials')
         self.assertTrue(alfio.check_dimensions(trials) == 0)
-        extract_all(self.training_lt5['path'])
-        trials = alfio.load_object(self.training_lt5['path'] / 'alf', object='trials')
-        self.assertTrue(alfio.check_dimensions(trials) == 0)
         extract_all(self.biased_ge5['path'])
         trials = alfio.load_object(self.biased_ge5['path'] / 'alf', object='trials')
         self.assertTrue(alfio.check_dimensions(trials) == 0)
-        # Wheel moves extraction fails for these wheel data; skipping
-        # extract_training(self.biased_lt5['path'])
-        # trials = alf.io.load_object(self.biased_lt5['path'] / 'alf', object='_ibl_trials')
-        # self.assertTrue(alf.io.check_dimensions(trials) == 0)
+        # VERSION < 5.0.0
+        # for these test data there are no wheel moves so let's mock the output
+        mock_data = {
+            'intervals': np.array([[0, 1], ]),
+            'peakAmplitude': np.array([1, 1]),
+            'peakVelocity_times': np.array([1, 1])}
+        function_name = 'ibllib.io.extractors.training_wheel.extract_wheel_moves'
+        # Training
+        with unittest.mock.patch(function_name, return_value=mock_data):
+            extract_all(self.training_lt5['path'])
+        trials = alfio.load_object(self.training_lt5['path'] / 'alf', object='trials')
+        self.assertTrue(alfio.check_dimensions(trials) == 0)
+        # Biased
+        with unittest.mock.patch(function_name, return_value=mock_data):
+            extract_all(self.biased_lt5['path'])
+        trials = alfio.load_object(self.biased_lt5['path'] / 'alf', object='trials')
+        self.assertTrue(alfio.check_dimensions(trials) == 0)
 
     def tearDown(self):
         for f in self.main_path.rglob('_ibl_log.*.log'):
@@ -633,10 +637,10 @@ class MockExtracor(BaseExtractor):
     )
 
     def _extract(self, **kwargs) -> tuple:
-        csv = pd.DataFrame([])
-        ssv = pd.DataFrame([])
-        tsv = pd.DataFrame([])
-        npy = np.array([])
+        csv = pd.DataFrame([1, 2, 3])
+        ssv = pd.DataFrame([1, 2, 3])
+        tsv = pd.DataFrame([1, 2, 3])
+        npy = np.array([1, 2, 3])
 
         return (csv, ssv, tsv, npy)
 
