@@ -6,13 +6,13 @@ import logging
 import numpy as np
 import matplotlib
 import matplotlib.pyplot as plt
-
 from one.api import ONE
+from iblutil.util import Bunch
+
 from ibllib.tests import TEST_DB
-from ibllib.qc.camera import CameraQC
+from ibllib.qc.camera import CameraQC, get_task_collection
 from ibllib.io.raw_data_loaders import load_camera_ssv_times
 from ibllib.tests.fixtures import utils
-from iblutil.util import Bunch
 
 
 class TestCameraQC(unittest.TestCase):
@@ -279,6 +279,15 @@ class TestCameraQC(unittest.TestCase):
         with self.assertRaises(AssertionError):
             self.qc.ensure_required_data()
 
+    def test_get_task_collection(self):
+        """Test for ibllib.qc.camera.get_task_collection"""
+        params = {'version': '1.0.0', 'tasks': [
+            {'passiveChoiceWorld': {'collection': 'raw_task_data_00'}},
+            {'ephysChoiceWorld': {'collection': 'raw_task_data_01'}}
+        ]}
+        self.assertEqual('raw_behavior_data', get_task_collection(None))
+        self.assertEqual('raw_task_data_01', get_task_collection(params))
 
-if __name__ == "__main__":
+
+if __name__ == '__main__':
     unittest.main(exit=False, verbosity=2)
