@@ -5,7 +5,7 @@ import matplotlib.pyplot as plt
 
 from ibllib.atlas import (BrainCoordinates, cart2sph, sph2cart, Trajectory,
                           Insertion, ALLEN_CCF_LANDMARKS_MLAPDV_UM, AllenAtlas)
-from ibllib.atlas.regions import BrainRegions
+from ibllib.atlas.regions import BrainRegions, FranklinPaxinosRegions
 from ibllib.atlas.plots import prepare_lr_data, reorder_data
 from iblutil.numerical import ismember
 
@@ -284,6 +284,37 @@ class TestBrainRegions(unittest.TestCase):
             self.brs.parse_acronyms_argument(acronyms + ['toto'])
         assert np.all(self.brs.parse_acronyms_argument(acronyms + ['toto'], mode='clip') == ids)
 
+    def test_compute_hierarchy(self):
+        self.brs.compute_hierarchy()
+        np.testing.assert_equal(
+            self.brs.hierarchy[:, 349], np.array([2, 3, 4, 5, 6, 340, 347, 349, 349, 349]))
+
+    def test_propagate_down(self):
+        acronyms = ['CB', 'ANcr2']
+        ind0 = ismember(self.brs.acronym2id(acronyms), self.brs.id)[1]
+        ind1 = [7, 45, 66]
+        for ind in [ind0, ind1]:
+            with self.subTest(indices=ind):
+                acronyms = self.brs.acronym[ind]
+                values = np.arange(len(acronyms))
+                ibr, ivals = self.brs.propagate_down(acronyms, values)
+                idesc = []
+                for c, i in enumerate(ind):
+                    ii = ismember(self.brs.descendants(self.brs.id[i])['id'], self.brs.id)[1]
+                    idesc.append(ii)
+                np.testing.assert_equal(ibr, np.unique(np.concatenate(idesc)))
+
+
+class TestAtlas(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(self):
+        self.ba = _create_mock_atlas()
+
+    def test_compute_regions_volume(self):
+        self.ba.compute_regions_volume()
+        self.assertTrue(self.ba.regions.volume.shape == self.ba.regions.acronym.shape)
+
 
 class TestAtlasPlots(unittest.TestCase):
 
@@ -355,6 +386,10 @@ class TestAtlasSlicesConversion(unittest.TestCase):
         assert (np.all(aids == np.array([0])))
         assert (np.isclose(proportions, np.array([1.])))
 
+    def test_plot_slices(self):
+        axs = self.ba.plot_slices(np.array([0, -.0024, -.0038]))
+        assert axs.shape == (2, 2)
+
     def test_slice(self):
         ba = self.ba
         nx, ny, nz = ba.bc.nxyz
@@ -597,5 +632,48 @@ class TestsCoordinatesSimples(unittest.TestCase):
         assert np.all(np.isclose(dv, z_))
 
 
+class TestFranklinPaxinos(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(self):
+        self.frs = FranklinPaxinosRegions()
+        self.brs = BrainRegions()
+
+    def test_level(self):
+
+        # This is a region that is only in FranklinPaxinos Atlas
+        info_fr5 = self.frs.get(2325)
+        # This is a region that is in both Allen and Franklin that should share parent with above region
+        info_fr23 = self.frs.get(667)
+        np.testing.assert_equal(info_fr5['level'], info_fr23['level'])
+
+        # These are the levels in the franklin paxinos
+        info_V1M6b = self.frs.get(2433)  # only in franklin
+        info_V1M = self.frs.get(2098)  # only in franklin
+        info_V1 = self.frs.get(385)  # in frankin and in Allen
+
+        # equivalent of V1 in Allen
+        info_VISp = self.brs.get(self.brs.acronym2id('VISp'))
+
+        np.testing.assert_equal(info_V1['level'], info_VISp['level'])
+        np.testing.assert_equal(info_V1M['level'], info_V1['level'] + 1)
+        np.testing.assert_equal(info_V1M6b['level'], info_V1['level'] + 2)
+
+    def test_conversions(self):
+
+        # These are regions just in FP atlas
+        np.testing.assert_equal(self.frs.acronym2id('FrA-5'), 2325)
+        np.testing.assert_equal(self.frs.acronym2index('FrA-5')[1][0][0], 10)
+        np.testing.assert_equal(self.frs.acronym2index('FrA-5')[1][0][1], 1440)
+        np.testing.assert_equal(self.frs.acronym2id('V1M-6b'), 2433)
+        np.testing.assert_equal(self.frs.acronym2index('V1M-6b')[1][0][0], 209)
+        np.testing.assert_equal(self.frs.acronym2index('V1M-6b')[1][0][1], 1639)
+
+        # These are regions in both and so have taken on Allen acronyms
+        np.testing.assert_equal(self.frs.acronym2id('ORBl2/3'), 412)
+        np.testing.assert_equal(self.brs.acronym2id('ORBl2/3'), 412)
+        np.testing.assert_equal(self.frs.acronym2id('LO-2/3'), np.array([]))
+
+
 if __name__ == "__main__":
     unittest.main(exit=False, verbosity=2)
